#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifFancyListBox.tcl,v 1.1 2004/09/06 14:47:13 cguirao Exp $" 
#
# uifListBox.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.Comin    07/11/94   created
# M.COMIN    15/03/95   Man page updated 
# M.COMIN    20/03/95   Add bg,fg,label public variables
# M.COMIN    21/03/95   Add method : addItemList
# M.COMIN    07/04/95   Add switch Y scroll ON/OFF
# M.COMIN    20/04/95   Add method getList
# M.COMIN    31/07/95   Use args instead of config
# M.COMIN    10/08/95   Add check on current selection index
# M.COMIN    18/08/95   Add check on getSelection and getItem
# M.COMIN    02/10/95   Relief changed to sunken
#
# mcomin      6/03/96   Porting to Tk4.0 and itcl2.0
#            07/05/96   Fixed bug in setBind method
#            13/05/96   Fixed bug in setVariable method
#            15/05/96   Update man page
#
#            05/02/97   Several upgrades
#            16/02/97   Add multiple selection, method setSelection upgraded
#            06/03/97   Porting to TK 4.2
#            02/06/97   Add method moveto
#            24/06/97   Activate default binding by setting focus to uifListBox
#            04/04/97   CG. tkCancelRepeat is obsolete.
#
#************************************************************************
#   NAME
#     UifFancyListBox - Class for VLT listBox widget
#   SYNOPSIS
#     UifFancyListBox  <widget-name>
#   DESCRIPTION
#
#     UifFancyListBox defines the class for the VLT List Box widget. 
#     The widget also provides a vertical and horizontal scroll bars
#     which can be enabled/disabled through the associated widget options.
#
#     The user create a widget of this class (called instance) by :
#
#         UifFancyListBox  <widget-name> <config-parameters>
#   COMPONENTS
#
#     The itk 'component' method returns the component's full name.
#
#     scrollh    - Horizontal scroll bar
#     scrollv    - Vertical   scroll bar
#     uifFancyListBox - Listbox widget
#   METHODS
#
#    General remarks :
#
#    - Positions in the list start at element 0.
#    - keyword "end" identifies the last position of the listBox
#
#     addItem <pos> <text>
#        Adds the string specified in <text> at position <pos>
#
#     addItemList <pos> <list>
#        Adds the specified list of items from  position <pos>
#
#     clearSelection
#        Clears all selections
#
#     deleteItem <first> <last>
#        Removes from the list elements from <first> to <last>.
#        If <last> is omitted, only the element <first> is removed.
#
#     getIndex
#        Returns the index of the currently selected item.
#
#     getList
#        Returns the list of all elements.
#
#     getSize
#        Returns the total number of elements in the list.
#
#     getItem <pos>
#        Returns list elements in position <pos>
#        If <pos> is a wrong number, it return an empty string ("").
#
#     getSelection   
#        Returns the currently selected item.
#        If no element are selected or the listBox is empty it returns
#        an empty string ("")
#
#     moveto <h/v> <pos>
#        Shows the <pos> percentage of the listbox along h=horizontal or 
#        v=vertical direction :  <pos> must be in the range of 0 and 1, 
#        being 0 the top/left-side and 1 the bottom/right-side.
#
#     setBind <key> <action>
#        Allows the user to define specific binding.
#
#     setSelection <pos>
#        Sets the selection to element <pos> and returns the value of the 
#        selected element(s) : selected elements are highlighted.
#
#        If the selection mode is not 'single' more than one element can 
#        be selected : if <POS> = ALL all elements are selected.
#
#     updateList <array> <formatting_procedure>
#        Initializes the content of the listBox with the content of the <array>
#        Each element of the <array> becomes an entry of the listBox and
#        it is displayed according to the format returned by 
#        <formatting_procedure> (see example)
#
#     updateSortList <order> <type> <array> <formatting_procedure>
#        Perform the same function described for 'updateList', but the <array>
#        is first sorted according to the following flags :
#
#        <order>    Specifies the ordering as : "-decreasing" or"-increasing"
#
#        <type>     The type of data to be sorted : "-ascii", "-integer" or
#                   "-real"
#
#        If <order> <type> are set to "", the following default values
#        are used :  "-increasing", "-ascii"
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifInteractiveWidget class and supports :
#
#     font      <font name>
#     geometry  <width>x<height> 
#     itemList  <list>            List of elements.
#     label     <string>          Give a title to the listbox
#     scrollx   <1/0>             Enables X scrolling : default is 1.
#     scrolly   <1/0>             Enables Y scrolling : default is 1.
#     selectmode                  Valid values : single/browse/multiple
#     variable  <var. Name>       Name of the variable where to store
#                                 the selection.
#   SPECIFIC BINDINGS
#
#     <Button-1>     Selects an element of the list.
#                    The selected element is stored into the variable specified
#                    by widget option 'variable'.
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifFancyListBox .w  -label myListBox -variable gvar(myItem) \
#                    -geometry 10x5 -scrollx 1
# 
#     Set Bindings 
#
#        .w setBind <B1-ButtonRelease>  "myProcedure"
# 
#     Add and Delete Elements
#    
#        set myList { item1 item2 item3 }
#
#        .w addItemList end $myList
#
#        Another alternative
#
#        foreach item $itemList {
#          .w addItem end $myList   
#        }
#
#     The following table summarizes the widget behaviour when the use
#     uses the 'setSelection' method to choose elements, for example, when
#     the widget is configured as follows :
#
#          UifFancyListBox .l -itemList { a b "c d" e f g } -variable tmp
#
#         selectmode    |   statement             |   tmp contains
#     ______________________________________________________________________
#           single      | .w setSelection 2       | "c d"
#                       | .w setSelection 5       |  g
#                       | .w setSelection 2 3 5   |  g (the last element)
#                       | .w setSelection ALL     |  g ( tmp does not change)
#                       |                         | 
#       browse/multiple | .w setSelection 2       |  {c d}
#                       | .w setSelection 2 3 5   |  {c d} e g
#                       | .w setSelection ALL     |  a b {c d} e f g
#
#     Examples using arrays : In this example each element of the array
#     contains the information on a country : continent, area.
#
#     set array(japan)  "Asia  144"
#     set array(china)  "Asia  1247"
#     set array(Brazil) "{South America} 3286"
#     set array(France) "Europe 211"
#     
#     proc myFormat { text } {        
#        set country [lindex $text 0]
#        set pop     [lindex $text 1]
#        set newText [format "%-20s  %5d" $country $pop]
#        return $newText
#     }   
#
#     List items are displayed with no format
#
#       .w updateList array ; # List items are displayed with no format
#       .w deleteItem 0 end ; # Clear the list Box
#
#     List elements are displayed according to the format
#
#       .w updateList array myFormat
#
#     List elements are sorted and displayed according to the format
#
#       .w updateSortList "-decreasing" "-ascii" array myFormat
#
#   SEE ALSO
#     UifIntercativeWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

option add *UifFancyListBox.relief sunken widgetDefault
option add *UifFancyListBox.borderwidth 2 widgetDefault
option add *UifFancyListBox.selectmode single widgetDefault

class UifFancyListBox {
    inherit UifInteractiveWidget

    constructor {args} {}

    itk_option define -variable  variable  Variable   ""
    itk_option define -scrollx   scrollx   Scrollx    1
    itk_option define -scrolly   scrolly   Scrolly    1
    itk_option define -geometry  geometry  Geometry   "5x5"
    itk_option define -label     label     Label      ""
    itk_option define -itemList  itemList  ItemList   {}    

    public method getSelection   {}
    public method getItem        {pos}
    public method deleteItem     {first {last {}} } 
    public method getList        {}
    public method addItemList    {pos listOfItems}
    public method updateList     {array {args } }
    public method updateSortList { order type  array {args } } 
    public method setVariable    {var}
    public method setBind        {key action}
    public method setSelection   {pos}
    public method clearSelection {}
    public method setCursor      {pos}
    public method moveto         {what pos}

    public method getSize  {} {return [$itk_component(uifFancyListBox) size] }
    public method getIndex {} {return [$itk_component(uifFancyListBox) curselection ]}
    public method addItem {pos text} {$itk_component(uifFancyListBox) insert $pos $text}  
    
    private method writeElements {nameList array}
    private method checkPos      {pos}

    private variable formatProc ""
    private variable packLabel  0
    private variable bindFlag   0

}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifFancyListBox::constructor {args} {

   itk_component add f1 {
      frame  $itk_interior.f1 
   }

   itk_component add f2 {
      frame  $itk_interior.f2 
   }

   itk_component add label {
      label  $itk_interior.label
   } {
      keep -font
   }

   pack $itk_component(f1) -fill both -expand 1
   pack $itk_component(f2) -fill x    

   itk_component add uifFancyListBox {
       #listbox $itk_component(f1).uifFancyListBox
       fancylistbox $itk_component(f1).uifFancyListBox
   } { 
       #keep -relief -font -yscrollcommand -xscrollcommand -borderwidth \
       #    -width -height -background -foreground -selectbackground \
       #    -selectforeground -selectmode
       
       keep -wrap -cursor -selectmode -relief -height -background
   }
   
   itk_component add scrollv {
      scrollbar $itk_component(f1).scrollv -orient v -width 10 -relief sunken\
                -command [code $itk_component(uifFancyListBox) yview] 
   } { 
      keep -borderwidth -activebackground
   }

   itk_component add scrollh {
      scrollbar $itk_component(f2).scrollh -orient h -width 10 -relief sunken\
                -command [code $itk_component(uifFancyListBox) xview]
   } { 
      keep -borderwidth -activebackground
   }

   pack $itk_component(uifFancyListBox)  -side left -fill both -expand 1 
   pack $itk_component(scrollv)     -side left -fill y  
   pack $itk_component(scrollh)     -fill x
   
   eval itk_initialize 
   set initialized 1

   # Set the focus on the listbox in order to acivate the default bindings
   bind $itk_component(uifFancyListBox) <1> "focus $itk_component(uifFancyListBox)"

   $itk_component(uifFancyListBox) configure \
                        -yscrollcommand [code $itk_component(scrollv) set] \
	                -xscrollcommand [code $itk_component(scrollh) set]

   eval configure -selectmode single $args
}    

#---------------------------------------------------------------
# METHOD : moveto - Set the h/v scrollbar to the percentage 
#                   indicated by what
#---------------------------------------------------------------

body UifFancyListBox::moveto {what pos} {
   switch $what {
      h       {$itk_component(uifFancyListBox) xview moveto $pos }
      v       {$itk_component(uifFancyListBox) yview moveto $pos }
      default { puts "Invalid 'what' for method UifFancyListBox::moveto" }
   }
}

#---------------------------------------------------------------
# METHOD : setBind 
#---------------------------------------------------------------

body UifFancyListBox::setBind {key action} {
   
#    puts "DBG> OLD  bindtags listBox [bindtags $itk_component(uifFancyListBox)]"
    bind ${this}${key} $key "$action" 

    # Add new bindtag ... if it does not already appear on the list

    set tmp [ bindtags $itk_component(uifFancyListBox)]

    if {[lsearch $tmp ${this}${key}] == -1 } {
       bindtags $itk_component(uifFancyListBox) \
	     [linsert [bindtags $itk_component(uifFancyListBox)] end ${this}${key}]
    }
#    puts "DBG> NEW  bindtags listBox [bindtags $itk_component(uifFancyListBox)]"
}
  
  
#---------------------------------------------------------------
# METHOD : getSelection
#---------------------------------------------------------------

#body UifFancyListBox::getSelection {} {
#   set tmp ""
#   if { [set index [$list curselection]] != "" } {
#      foreach item $index {
#         lappend tmp [$list get $item]
#      }
#   }
#   return $tmp
#}

body UifFancyListBox::getSelection {} {
   set index [$itk_component(uifFancyListBox) curselection]

   if {[cequal $index ""] } { return "" }

   if {[cequal [$this cget -selectmode] single]} {
       return [$itk_component(uifFancyListBox) get $index]
   } else {
       foreach item $index {
          lappend tmp [$itk_component(uifFancyListBox) get $item]
       }
       return $tmp
   }
}


#---------------------------------------------------------------
# METHOD : checkPos - If the position is not valid it returns 
#                     an empty string                
#---------------------------------------------------------------
           
body UifFancyListBox::checkPos {pos} {
   if {[cequal $pos end] || $pos == 0 } { return $pos }
   if { ( $pos >= 0 && $pos < [$itk_component(uifFancyListBox) size]) } { 
      return $pos 
   }
   return ""
}

#---------------------------------------------------------------
# METHOD : getItem
#---------------------------------------------------------------
           
body UifFancyListBox::getItem {pos} {
   if {[cequal [checkPos $pos] ""]} { 
      return ""
   } else {
      return [$itk_component(uifFancyListBox) get $pos] 
   } 
}

#---------------------------------------------------------------
# METHOD : getList
#---------------------------------------------------------------

body UifFancyListBox::getList {} { 
   set myList {}
   set max    [$itk_component(uifFancyListBox) size]

   for {set i 0} {$i < $max} { incr i } {
       lappend myList [$itk_component(uifFancyListBox) get $i]  
   }
   return $myList
}    
   
#---------------------------------------------------------------
# METHOD : deleteItem
#---------------------------------------------------------------
       
body UifFancyListBox::deleteItem {first {last {}} } {
   if {![cequal $last ""]} {
      $itk_component(uifFancyListBox) delete $first $last
   } else {
      $itk_component(uifFancyListBox) delete $first 
   }        
}  
  
#---------------------------------------------------------------
# METHOD : clearSelection
#---------------------------------------------------------------
           
body UifFancyListBox::clearSelection {} {
   $itk_component(uifFancyListBox) selection clear 0 end
}       


#---------------------------------------------------------------
# METHOD : setSelection
#---------------------------------------------------------------
           
body UifFancyListBox::setSelection { args } {

   if {[cequal $args ALL]} {

      # Select ALL elements only if '-selectmode' is not se to 'single'
      if {[cequal [$this cget -selectmode] single]} { return }
      $itk_component(uifFancyListBox) selection set 0 end

   } else {

      # If the listBox has set the '-selectmode' to 'single', at most
      # one element can be active at the time.

      foreach i $args {
         if {[checkPos $i] != ""} { 
            if {[cequal [$this cget -selectmode] single]} {
               clearSelection  ;# clear previous elements
            }

            $itk_component(uifFancyListBox) selection set $i
         }
      }
      $itk_component(uifFancyListBox) see $i
   }
   
   if {![cequal $itk_option(-variable) ""]} {
      setVariable $itk_option(-variable)
   }
}    
       

# This is here for backward compatibility ..
body UifFancyListBox::setCursor { pos } {
   setSelection $pos
}
 
#---------------------------------------------------------------
# METHOD : addItemList
#---------------------------------------------------------------
 
body UifFancyListBox::addItemList { pos listOfItems } {

   if {[cequal [checkPos $pos] ""]} { return }

   if {[cequal $pos end] } {
      foreach item $listOfItems {
         $itk_component(uifFancyListBox) insert end $item
      }
   } else {
      set len [llength $listOfItems]
       for { set i [expr {$len-1}]} {$i >=0} {incr i -1}  {
          $itk_component(uifFancyListBox) insert $pos [lindex $listOfItems $i] 
      }  
   }                
} 
 
#---------------------------------------------------------------
# METHOD : updateList
#---------------------------------------------------------------

body UifFancyListBox::updateList { array {args } } {
   upvar $array arrayName
   
   if { [llength $args] } { 
      set formatProc [lindex $args 0] 
   } else {
      set formatProc ""
   }
   writeElements [array names arrayName]  arrayName          
}   

#---------------------------------------------------------------
# METHOD : updateSortList
#---------------------------------------------------------------

body UifFancyListBox::updateSortList { order type  array {args } } {
   upvar $array arrayName

   if { [llength $args] } { 
      set formatProc [lindex $args 0] 
   } else {
      set formatProc ""
   }

   if {[cequal $order ""] } { set order "-increasing" }
   if {[cequal $type  ""] } { set type  "-ascii" }

   writeElements [lsort $order $type [array names arrayName]] arrayName 
}

#---------------------------------------------------------------
# METHOD : setVariable

# The statement list; is used to avoid complaints when the value
# contains special characters such as : "" and $.

#---------------------------------------------------------------
           
body UifFancyListBox::setVariable { var } {
   uplevel #0 set $var [list [$this getSelection]]
} 
 
#---------------------------------------------------------------
# METHOD : writeElements
#---------------------------------------------------------------

body UifFancyListBox::writeElements { nameList array } {
   upvar $array arrayName
   $itk_component(uifFancyListBox) delete 0 end

   foreach name $nameList {
      if { $formatProc != "" } {   
         set newText    [ eval { $formatProc $arrayName($name) }]
         $itk_component(uifFancyListBox)  insert end  $newText        
      } else {
         $itk_component(uifFancyListBox)  insert end  $arrayName($name)
      }
   }
}

#---------------------------------------------------------------
# OPTION : -scrollx
#---------------------------------------------------------------  

configbody UifFancyListBox::scrollx {    
   if { $itk_option(-scrollx) } {
      pack $itk_component(f2) -fill x
   } else {
      pack forget $itk_component(f2)   
   }
}

#---------------------------------------------------------------
# OPTION : -scrolly
#---------------------------------------------------------------  

configbody UifFancyListBox::scrolly {
   if {$itk_option(-scrolly)} {
      pack $itk_component(scrollv) -side left -fill y
   } else {
      pack forget $itk_component(scrollv)
   }
}
   
#---------------------------------------------------------------
# OPTION : -geometry
#---------------------------------------------------------------  

configbody UifFancyListBox::geometry {
   if {[scan $itk_option(-geometry) "%dx%d" w h] > 0} {
      $itk_component(uifFancyListBox) configure -width $w -height $h
   }
}
 
#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------  

configbody UifFancyListBox::variable {
   if { [cequal $itk_option(-variable) ""] } {
      if {$bindFlag} {
          setBind <B1-ButtonRelease> "" 
	  bind $itk_component(uifFancyListBox) <Return> {}
      }
   } else {

       # Added to set the variable when using default bindings to select
       # a list element

       bind $itk_component(uifFancyListBox) <Return> \
            "$wthis setVariable $itk_option(-variable)"

       setBind <B1-ButtonRelease> "$wthis setVariable $itk_option(-variable)" 
       set bindFlag 1
   }
}     

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------  

configbody UifFancyListBox::label {

   if {[cequal $itk_option(-label) ""] } {
      if {$packLabel} { 
         pack forget $itk_component(label)
         set packLabel 0
      }
   } else {
      $itk_component(label) configure -text $itk_option(-label)

      if { $packLabel == 0 } {
         pack $itk_component(label) -before $itk_component(f1) -fill x
         set packLabel 1
      }
   }
} 
 
#---------------------------------------------------------------
# OPTION : -itemList
#---------------------------------------------------------------  

configbody UifFancyListBox::itemList {
   $itk_component(uifFancyListBox) delete 0 end

   foreach item $itk_option(-itemList) {
       $itk_component(uifFancyListBox) insert end $item 
   }
}

# listbox.tcl --
#
# This file defines the default bindings for Tk listbox widgets
# and provides procedures that help in implementing those bindings.
#
# SCCS: @(#) listbox.tcl 1.16 96/04/16 11:42:22
#
# Copyright (c) 1994 The Regents of the University of California.
# Copyright (c) 1994-1995 Sun Microsystems, Inc.
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

#--------------------------------------------------------------------------
# tkPriv elements used in this file:
#
# afterId -		Token returned by "after" for autoscanning.
# listboxPrev -		The last element to be selected or deselected
#			during a selection operation.
# listboxSelection -	All of the items that were selected before the
#			current selection operation (such as a mouse
#			drag) started;  used to cancel an operation.
#--------------------------------------------------------------------------

#-------------------------------------------------------------------------
# The code below creates the default class bindings for listboxes.
#-------------------------------------------------------------------------

# Note: the check for existence of %W below is because this binding
# is sometimes invoked after a window has been deleted (e.g. because
# there is a double-click binding on the widget that deletes it).  Users
# can put "break"s in their bindings to avoid the error, but this check
# makes that unnecessary.

bind Listbox <1> {
    if {[winfo exists %W]} {
	tkListboxBeginSelect %W [%W index @%x,%y]
    }
}

# Ignore double clicks so that users can define their own behaviors.
# Among other things, this prevents errors if the user deletes the
# listbox on a double click.

bind Listbox <2> {
    # Empty script
}

bind Listbox <B1-Motion> {
    set tkPriv(x) %x
    set tkPriv(y) %y
    tkListboxMotion %W [%W index @%x,%y]
}
bind Listbox <ButtonRelease-1> {
    #tkCancelRepeat
    %W activate @%x,%y
}
bind Listbox <Shift-1> {
    tkListboxBeginExtend %W [%W index @%x,%y]
}
bind Listbox <Control-1> {
    tkListboxBeginToggle %W [%W index @%x,%y]
}
bind Listbox <B1-Leave> {
    set tkPriv(x) %x
    set tkPriv(y) %y
    tkListboxAutoScan %W
}
bind Listbox <B1-Enter> {
    #tkCancelRepeat
}

bind Listbox <Up> {
    tkListboxUpDown %W -1
}
bind Listbox <Shift-Up> {
    tkListboxExtendUpDown %W -1
}
bind Listbox <Down> {
    tkListboxUpDown %W 1
}
bind Listbox <Shift-Down> {
    tkListboxExtendUpDown %W 1
}
bind Listbox <Left> {
    %W xview scroll -1 units
}
bind Listbox <Control-Left> {
    %W xview scroll -1 pages
}
bind Listbox <Right> {
    %W xview scroll 1 units
}
bind Listbox <Control-Right> {
    %W xview scroll 1 pages
}
bind Listbox <Prior> {
    %W yview scroll -1 pages
    %W activate @0,0
}
bind Listbox <Next> {
    %W yview scroll 1 pages
    %W activate @0,0
}
bind Listbox <Control-Prior> {
    %W xview scroll -1 pages
}
bind Listbox <Control-Next> {
    %W xview scroll 1 pages
}
bind Listbox <Home> {
    %W xview moveto 0
}
bind Listbox <End> {
    %W xview moveto 1
}
bind Listbox <Control-Home> {
    %W activate 0
    %W see 0
    %W selection clear 0 end
    %W selection set 0
}
bind Listbox <Shift-Control-Home> {
    tkListboxDataExtend %W 0
}
bind Listbox <Control-End> {
    %W activate end
    %W see end
    %W selection clear 0 end
    %W selection set end
}
bind Listbox <Shift-Control-End> {
    tkListboxDataExtend %W end
}
bind Listbox <F16> {
    if {[selection own -displayof %W] == "%W"} {
	clipboard clear -displayof %W
	clipboard append -displayof %W [selection get -displayof %W]
    }
}
bind Listbox <space> {
    tkListboxBeginSelect %W [%W index active]
}
bind Listbox <Select> {
    tkListboxBeginSelect %W [%W index active]
}
bind Listbox <Control-Shift-space> {
    tkListboxBeginExtend %W [%W index active]
}
bind Listbox <Shift-Select> {
    tkListboxBeginExtend %W [%W index active]
}
bind Listbox <Escape> {
    tkListboxCancel %W
}
bind Listbox <Control-slash> {
    tkListboxSelectAll %W
}
bind Listbox <Control-backslash> {
    if {[%W cget -selectmode] != "browse"} {
	%W selection clear 0 end
    }
}

# Additional Tk bindings that aren't part of the Motif look and feel:

bind Listbox <2> {
    %W scan mark %x %y
}
bind Listbox <B2-Motion> {
    %W scan dragto %x %y
}

# tkListboxBeginSelect --
#
# This procedure is typically invoked on button-1 presses.  It begins
# the process of making a selection in the listbox.  Its exact behavior
# depends on the selection mode currently in effect for the listbox;
# see the Motif documentation for details.
#
# Arguments:
# w -		The listbox widget.
# el -		The element for the selection operation (typically the
#		one under the pointer).  Must be in numerical form.

proc tkListboxBeginSelect {w el} {
    global tkPriv

    if {[$w cget -selectmode]  == "multiple"} {
	if {[$w selection includes $el]} {
	    $w selection clear $el
	} else {
	    $w selection set $el
	}
    } else {
	$w selection clear 0 end
	$w selection set $el
	$w selection anchor $el
	set tkPriv(listboxSelection) {}
	set tkPriv(listboxPrev) $el
    }
}

# tkListboxMotion --
#
# This procedure is called to process mouse motion events while
# button 1 is down.  It may move or extend the selection, depending
# on the listbox's selection mode.
#
# Arguments:
# w -		The listbox widget.
# el -		The element under the pointer (must be a number).

proc tkListboxMotion {w el} {
    global tkPriv
    if {$el == $tkPriv(listboxPrev)} {
	return
    }
    set anchor [$w index anchor]
    switch [$w cget -selectmode] {
	browse {
	    $w selection clear 0 end
	    $w selection set $el
	    set tkPriv(listboxPrev) $el
	}
	extended {
	    set i $tkPriv(listboxPrev)
	    if {[$w selection includes anchor]} {
		$w selection clear $i $el
		$w selection set anchor $el
	    } else {
		$w selection clear $i $el
		$w selection clear anchor $el
	    }
	    while {($i < $el) && ($i < $anchor)} {
		if {[lsearch $tkPriv(listboxSelection) $i] >= 0} {
		    $w selection set $i
		}
		incr i
	    }
	    while {($i > $el) && ($i > $anchor)} {
		if {[lsearch $tkPriv(listboxSelection) $i] >= 0} {
		    $w selection set $i
		}
		incr i -1
	    }
	    set tkPriv(listboxPrev) $el
	}
    }
}

# tkListboxBeginExtend --
#
# This procedure is typically invoked on shift-button-1 presses.  It
# begins the process of extending a selection in the listbox.  Its
# exact behavior depends on the selection mode currently in effect
# for the listbox;  see the Motif documentation for details.
#
# Arguments:
# w -		The listbox widget.
# el -		The element for the selection operation (typically the
#		one under the pointer).  Must be in numerical form.

proc tkListboxBeginExtend {w el} {
    if {([$w cget -selectmode] == "extended")
	    && [$w selection includes anchor]} {
	tkListboxMotion $w $el
    }
}

# tkListboxBeginToggle --
#
# This procedure is typically invoked on control-button-1 presses.  It
# begins the process of toggling a selection in the listbox.  Its
# exact behavior depends on the selection mode currently in effect
# for the listbox;  see the Motif documentation for details.
#
# Arguments:
# w -		The listbox widget.
# el -		The element for the selection operation (typically the
#		one under the pointer).  Must be in numerical form.

proc tkListboxBeginToggle {w el} {
    global tkPriv
    if {[$w cget -selectmode] == "extended"} {
	set tkPriv(listboxSelection) [$w curselection]
	set tkPriv(listboxPrev) $el
	$w selection anchor $el
	if {[$w selection includes $el]} {
	    $w selection clear $el
	} else {
	    $w selection set $el
	}
    }
}

# tkListboxAutoScan --
# This procedure is invoked when the mouse leaves an entry window
# with button 1 down.  It scrolls the window up, down, left, or
# right, depending on where the mouse left the window, and reschedules
# itself as an "after" command so that the window continues to scroll until
# the mouse moves back into the window or the mouse button is released.
#
# Arguments:
# w -		The entry window.

proc tkListboxAutoScan {w} {
    global tkPriv
    if {![winfo exists $w]} return
    set x $tkPriv(x)
    set y $tkPriv(y)
    if {$y >= [winfo height $w]} {
	$w yview scroll 1 units
    } elseif {$y < 0} {
	$w yview scroll -1 units
    } elseif {$x >= [winfo width $w]} {
	$w xview scroll 2 units
    } elseif {$x < 0} {
	$w xview scroll -2 units
    } else {
	return
    }
    tkListboxMotion $w [$w index @$x,$y]
    set tkPriv(afterId) [after 50 tkListboxAutoScan $w]
}

# tkListboxUpDown --
#
# Moves the location cursor (active element) up or down by one element,
# and changes the selection if we're in browse or extended selection
# mode.
#
# Arguments:
# w -		The listbox widget.
# amount -	+1 to move down one item, -1 to move back one item.

proc tkListboxUpDown {w amount} {
    global tkPriv
    $w activate [expr {[$w index active] + $amount}]
    $w see active
    switch [$w cget -selectmode] {
	browse {
	    $w selection clear 0 end
	    $w selection set active
	}
	extended {
	    $w selection clear 0 end
	    $w selection set active
	    $w selection anchor active
	    set tkPriv(listboxPrev) [$w index active]
	    set tkPriv(listboxSelection) {}
	}
    }
}

# tkListboxExtendUpDown --
#
# Does nothing unless we're in extended selection mode;  in this
# case it moves the location cursor (active element) up or down by
# one element, and extends the selection to that point.
#
# Arguments:
# w -		The listbox widget.
# amount -	+1 to move down one item, -1 to move back one item.

proc tkListboxExtendUpDown {w amount} {
    if {[$w cget -selectmode] != "extended"} {
	return
    }
    $w activate [expr {[$w index active] + $amount}]
    $w see active
    tkListboxMotion $w [$w index active]
}

# tkListboxDataExtend
#
# This procedure is called for key-presses such as Shift-KEndData.
# If the selection mode isn't multiple or extend then it does nothing.
# Otherwise it moves the active element to el and, if we're in
# extended mode, extends the selection to that point.
#
# Arguments:
# w -		The listbox widget.
# el -		An integer element number.

proc tkListboxDataExtend {w el} {
    set mode [$w cget -selectmode]
    if {$mode == "extended"} {
	$w activate $el
	$w see $el
        if {[$w selection includes anchor]} {
	    tkListboxMotion $w $el
	}
    } elseif {$mode == "multiple"} {
	$w activate $el
	$w see $el
    }
}

# tkListboxCancel
#
# This procedure is invoked to cancel an extended selection in
# progress.  If there is an extended selection in progress, it
# restores all of the items between the active one and the anchor
# to their previous selection state.
#
# Arguments:
# w -		The listbox widget.

proc tkListboxCancel w {
    global tkPriv
    if {[$w cget -selectmode] != "extended"} {
	return
    }
    set first [$w index anchor]
    set last $tkPriv(listboxPrev)
    if {$first > $last} {
	set tmp $first
	set first $last
	set last $tmp
    }
    $w selection clear $first $last
    while {$first <= $last} {
	if {[lsearch $tkPriv(listboxSelection) $first] >= 0} {
	    $w selection set $first
	}
	incr first
    }
}

# tkListboxSelectAll
#
# This procedure is invoked to handle the "select all" operation.
# For single and browse mode, it just selects the active element.
# Otherwise it selects everything in the widget.
#
# Arguments:
# w -		The listbox widget.

proc tkListboxSelectAll w {
    set mode [$w cget -selectmode]
    if {($mode == "single") || ($mode == "browse")} {
	$w selection clear 0 end
	$w selection set active
    } else {
	$w selection set 0 end
    }
}




#___oOo___
