#!/bin/sh

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 1996 European Southern Observatory
# LICENSE
#
# NAME:      createCalibDB.sh
# TYPE:      Bourne Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    CG - ESO/DMD/SEG
# DESCRIPTION:
#            This script is used by 'quality' account during the DFS 
#            installation. It will check:
#                - The list of instrument packages available
#                - The list of instrument packages to be installed
#		 - The user is 'quality'
#                - Checks/creates the MSQL2 infrastructure
#                - Checks/creates the calibDB infrastructure
#                - Checks/starts the 'msql2d' daemon
#                - Drops/creates the complete calibDB 
#                - Creates the calibDB for each instrument package
#
# USAGE:     As 'quality' type:
#
#           	createCalibDB.sh  <instrument list>
#
#            where <instrument list> is any combination of:
#
#               Emmi Fors1 Fors2 Isaac Naco Sofi Susi Uves Vimos Vinci etc..
#               ALL: all instruments.
#
# EXAMPLES:
#            createCalibDB.sh Isacc Naco 
#            createCalibDBS.sh ALL
#
# $Id: createCalibDB.sh,v 1.1 2004/09/06 14:21:11 cguirao Exp $
#
# who            when            what
#------------------------------------------------------------------------------
# C. Guirao      29/11/02        created
#------------------------------------------------------------------------------

# Determine how to set prompt
if [ "`echo -n a`" = a ]; then
  # BSD echo
  NNN=-n
  CCC=
else
  # USG echo
  NNN=
  CCC='\c'
fi
readonly NNN CCC

#
# Print error message
#
error()
{
  echo '***' $* > /dev/tty
  false
}

#
#  A safe way to perform echo without sending a new-line
#
necho()
{
  echo $NNN "$*" $CCC > /dev/tty
}

#
#  Simply get the user to hit the return key
#
ask_return()
{
  necho 'Hit <return> to continue ...'
  read x
}

#
#  Get a yes/no answer from the user; $1 is the prompt, $2 the default
#
ask_yn()
{
  prompt="$1"
  def="${2:-y}"           # yes is the default if not supplied
  while true; do
    necho "$prompt [$def]: "
    read answer
    answer="${answer:-$def}"
    case "$answer" in
      [yY])
        answer=y
        break
        ;;
      [nN])
        answer=n
        break
        ;;
      *)
        error Please enter "'y'" or "'n'".
        ;;
    esac
  done
  [ $answer = y ]
}

#
#  Get a (non-zero) response from the user
#  $1 is the question, $2 is the default (if any).
#
ask_response()
{
  question="$1"
  default="$2"
  while true; do
    if [ -z "$default" ]; then
      necho "$question: "
    else
      necho "$question [$default]: "
    fi
    read answer
    if [ -z "$answer" ]; then
      answer=$default
    fi
    if [ -z "$answer" ]; then
      error Please enter a non-empty string.
    else
      break
    fi
  done
  echo $answer
}

#
# Find the list of packages (those in $DFS_HOME/sql/qucSql/*.sql
# and choose for the list only those that are readable
#
find_list()
{
  if [ ! -d  ${DFS_HOME}/sql/qucSql ]; then
    echo "Error: Cannot find ${DFS_HOME}/sql/qucSql directory. Exiting"
    exit 1
  fi
  for file in `ls ${DFS_HOME}/sql/qucSql/*.sql | grep -v CalibDb.sql` 
  do 
    if [ -f $file -a -r $file ] ; then
        instrument=`basename $file .sql`
	available_instruments="$available_instruments $instrument"
    fi
  done
}
#
#
# Check the list
#
check_list()
{
while [ -n "$1" ]
do
  if [ "$1" = "ALL" ]; then
    list=""
    check_list $available_instruments
    break 
  else
    unset found
    for instrument in $available_instruments
    do 
      if [ "$1" = "$instrument" ]; then
        list="$list $1"
        found=1
        shift 
      fi
    done
    if [ -z "$found" ] ; then
      echo""
      echo "Package $1 is not available" 
      echo $usage
      exit 1
    fi 
  fi
done
}

#
# Install the list
#
install_list()
{
if [ ! -f ${DFS_HOME}/sql/qucSql/CalibDb.sql ]; then
  echo ""
  echo "Error: file ${DFS_HOME}/sql/qucSql/CalibDb.sql does not exist"
  exit 1
fi

while [ -n "$1" ]
do
  echo ""
  echo Installing: $1
  echo "/opt/msql2/bin/msql calibDB < ${DFS_HOME}/sql/qucSql/$1.sql"
  /opt/msql2/bin/msql calibDB < ${DFS_HOME}/sql/qucSql/$1.sql  >/dev/null
  if [ "$?" != "0" ] ; then
    echo "Error in last command. Exit"
    exit 1
  fi
  shift
done
}

#
# Main
#
cmd=`basename $0`
available_instruments="";
list="";

#
# Check some environments
#
if [ -z "$DFS_HOME" ]; then
  echo ""
  echo "ERROR: DFS_HOME environment variable does not exist. Exiting"
  echo "Hint: DFS_HOME is defined by sourcing ~flowmgr/dfs/admin/dataflowUser.sh" 
  exit 1
fi

#
# Find the list of available instruments
#
find_list;
usage="Usage: $cmd [$available_instruments ALL]"

#
# This needs at least one argument
#
if [ -z "$1" ]; then
  echo $usage
  exit 1
fi

#
# Check the list of arguments
#
check_list $*
echo "List of packages to be installed:"
echo ""
echo " $list"
echo ""
if ! ask_yn "Is this list Ok? " ; then
  echo "Ok. Bye"
  exit 1
fi

# Check user, it should be 'quality' but let him go if s/he wish so
#
if [ `whoami` != "quality" ]; then
  echo ""
  echo "Warning: This script is meant to be executed by user 'quality'..."
  if ask_yn "nevertheless... do you want to continue?" "n"; then 
    echo "Ok, here we go..."
  else
    echo "Well done. Bye"
    exit 1
  fi
fi

#
# Creating the MSQL infrastructure if it does not exist.
#
mkdir -p ~/MSQL2/msqldb/.tmp
mkdir -p $DFS_DATA_ROOT/calibDB/certif/ins
mkdir -p $DFS_DATA_ROOT/calibDB/check/ins
mkdir -p ~/qc1/Dictionary

#
# Checking /MSQL2 as a soft link
#
if [ ! -h /MSQL2 -o ! -d /MSQL2 ]; then
  echo "Error: /MSQL2 is not a soft link or not pointing to a real directory"
  echo "Hint: As root execute"
  echo "  cd /"
  echo "  ln -s ~quality/MSQL2 /MSQL2"
  echo "Exit"
  exit 1
fi

#
# Checking /cal and /tcal as soft links
#
if [ ! -h /cal -o ! -d /cal ]; then
  echo "Error: /cal is not a soft link or not pointing to a real directory"
  echo "Hint: As root execute"
  echo "  cd /"
  echo "  ln -s $DFS_DATA_ROOT/calibDB/certif/ins /cal"
  echo "Exit"
  exit 1
fi
if [ ! -h /tcal -o ! -d /tcal ]; then
  echo "Error: /tcal is not a soft link or not pointing to a real directory"
  echo "Hint: As root execute"
  echo "  cd /"
  echo "  ln -s $DFS_DATA_ROOT/calibDB/check/ins /tcal"
  echo "Exit"
  exit 1
fi

#
# Checking /MSQL2 is pointing to ~quality/MSQL2
#
MSQL2_dir=`ls -l /MSQL2 | awk '{print $11}' `
expected_MSQL2_dir=`ls -ld ~quality/MSQL2 | awk '{print $9}' `
if [ "$MSQL2_dir" !=  "$expected_$MSQL2_dir" ]; then
  echo ""
  echo "Warning: /MSQL2 is not a soft link to $expected_MSQL2_dir, but to $MSQL2_dir"
  if ask_yn "Do you want to continue?" "n"; then
    echo "Ok. Let's continue..."
  else
    echo "Well done. Bye then.."
    exit 1
  fi
fi

#
# Checking /cal is pointing to $DFS_DATA_ROOT/calibDB/certif/ins
#
cal_dir=`ls -l /cal | awk '{print $11}' `
expected_cal_dir=`ls -ld $DFS_DATA_ROOT/calibDB/certif/ins | awk '{print $9}' `
if [ "$cal_dir" !=  "$expected_cal_dir" ]; then
  echo ""
  echo "Warning: /cal is not a soft link to $expected_cal_dir, but to $cal_dir"
  if ask_yn "Do you want to continue?" "n"; then
    echo "Ok. Let's continue..."
  else
    echo "Well done. Bye then.."
    exit 1
  fi
fi

#
# Checking /tcal is pointing to $DFS_DATA_ROOT/calibDB/check/ins
#
tcal_dir=`ls -l /tcal | awk '{print $11}' `
expected_tcal_dir=`ls -ld $DFS_DATA_ROOT/calibDB/check/ins | awk '{print $9}' `
if [ "$tcal_dir" !=  "$expected_tcal_dir" ]; then
  echo ""
  echo "Warning: /tcal is not a soft link to $expected_tcal_dir, but to $tcal_dir"
  if ask_yn "Do you want to continue?" "n"; then
    echo "Ok. Let's continue..."
  else
    echo "Well done. Bye then.."
    exit 1
  fi
fi

#
# Installation of an empty database
#
echo ""
if ask_yn "Ok to update or recreate calibDB from scratch"; then
  echo ""
  echo "Checking process msql2d is running..."
  msql_process=`ps -ef | grep msql2d| grep -v grep`
  if [ -z "$msql_process" ]; then
    echo "msql2d daemon not running... yet."
    if ask_yn "Ok to launch the msql2 daemon?"; then
      if msql2d=`which msql2d 2>/dev/null` ; then
         if [ "$msql2d" = '/opt/msql2/bin/msql2d' ]; then 
           /opt/msql2/bin/msql2d 2>/dev/null &
           sleep 1
           msql_process=`ps -ef | grep msql2d | grep -v grep`
           if [ -z "$msql_process" ]; then
             echo "msql2d daemon not running. Exiting"
             exit 1
           else
             echo "msql2d daemon running."
           fi
         else
           echo "Error: $msql2d daemon not in standard /opt/msql2/bin/msql2d location. Exiting"
           exit 1
         fi
      else
        echo "Error: msql2d daemon not found. Exiting"
        echo "       Was the msql package installed properly?"
        echo "To install msql as root, execute:"
        echo "   cd /opt"
        echo "   gunzip -c ${DFS_ROOT}/external-<re>/msql-<rel>-<OS>.tar.gz. | tar xf -"
        exit 1
      fi
    else
      echo "Sorry, but we need the process msql2d to continue"
      exit 1
    fi
  else
    echo "Yes! msql2d is running"
  fi
  if ask_yn "Ok to drop current calibDB and recreate it from scratch"; then
    echo ""
    echo "Installing an empty database..."
    echo "/opt/msql2/bin/msqladmin drop calibDB"
    /opt/msql2/bin/msqladmin drop calibDB 
    echo "/opt/msql2/bin/msqladmin create calibDB"
    /opt/msql2/bin/msqladmin create calibDB
    echo "/opt/msql2/bin/msql calibDB < ${DFS_HOME}/sql/qucSql/CalibDb.sql"
    /opt/msql2/bin/msql calibDB < ${DFS_HOME}/sql/qucSql/CalibDb.sql >/dev/null
    if [ "$?" != "0" ] ; then
      echo "Error in last command. Exit "
      exit 1
    fi
  else
    if ! ask_yn "Ok to only update calibDB with list: $list"; then
      echo "Then, let's stop here"
      exit
    fi
  fi
  echo ""
  echo "Installing list=$list"
  install_list $list
else
  echo "Then bye."
  exit 1
fi
exit 0
