package org.eso.gasgano.cpl;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.SwingUtilities;

import org.eso.cpl.Frame;
import org.eso.cpl.FrameGroup;
import org.eso.cpl.FrameLevel;
import org.eso.cpl.FrameList;
import org.eso.cpl.FrameType;
import org.eso.cpl.Recipe;
import org.eso.cpl.Request;
import org.eso.cpl.gui.CPLFrame;
import org.eso.cpl.jni.LibraryLoader;
import org.eso.gasgano.datamodel.gui.KeywordContainerOBComponent;
import org.eso.gasgano.datamodel.gui.OBComponent;

/**
 * @author Nick Kornweibel
 * 
 * A collection of recipe panels. It is responsible for managing the set of CPLFrame
 * objects displayed by Gasgano as well as for naming the panels uniquely.
 * PropertyChangeListeners may be added to be notified when panels are added or
 * removed.
 */

public class RecipePanelList {
    
    /**
     * Map mapping panel names to panels.
     */
    private Map panels = new HashMap();
    
    /**
     * List of current property change listeners.
     */
    private List listeners = new ArrayList();
    
    /**
     * Constructor for RecipePanelList.
     */
    public RecipePanelList() {
        super();
    }
    
    /**
     * Show the Recipe panel with the specified recipe, using the selected
     * frames as input frames in the recipe panel.
     * @param recipe
     */
    public void addRecipePanel(Recipe recipe, List obComponents, LibraryLoader loader) {
        String panelName = getNewName(recipe.getName()+" v"+recipe.getVersion());
        final CPLFrame cf = new CPLFrame(recipe, panelName);
        panels.put(panelName, cf);
        // notify listeners that the panels property has changed.
        // this is not done exactly to bean specification - the
        // panel name going from null to valid to null is meant to indicate
        // that a panel was added.
        firePropertyChange("panels", panelName, null);
        cf.setLibraryLoader(loader);
        Request req = new Request(recipe);
        req.getInputFrames().addAll(getRecipeInputFrames(obComponents));
        cf.setSingleRequest(req);
        cf.setVisible(true);
        cf.toFront();
        cf.addWindowListener(new WindowAdapter() {
            public void windowClosed(WindowEvent e) {
                String pName = cf.getPanelName();
                panels.remove(pName);
                // notify listeners that panels property has changed.
                // this is not done exactly to bean specification - the
                // panel name going from valid to null is meant to indicate
                // that a panel was removed.
                firePropertyChange("panels", pName, null); 
            }
            public void windowClosing(WindowEvent e) {
                String pName = cf.getPanelName();
                panels.remove(pName);
                // notify listeners that panels property has changed.
                // this is not done exactly to bean specification - the
                // panel name going from valid to null is meant to indicate
                // that a panel was removed.
                firePropertyChange("panels", pName, null); 
            }
        });  
    }
    
    /**
     * Generate a unqie name based in the base parameter (typically the recipe
     * name). This name is used to name the panel.
     * @param base
     * @return String
     */
    private String getNewName(String base) {
        int idx = 2;
        String rc = base;
        while (panels.get(rc) != null) {
            rc = base+" ("+idx+")";
            idx++;
        }
        return rc;
    }
    
    /**
     * Get the OBComponent selection in the file system tree view
     * and add this as the default frame selection for the recipe panel.
     * @return MarkedFrameList
     */
    public static FrameList getRecipeInputFrames(List obComponents) {
        
        FrameList inFrames = new FrameList();
        for (int i = 0; i < obComponents.size(); i++ ) {
            OBComponent obc = (OBComponent)obComponents.get(i);
            // if it is a fits file of tfits file we want it...
            if (obc instanceof KeywordContainerOBComponent) {
                KeywordContainerOBComponent ff = 
                    (KeywordContainerOBComponent)obc;
                Frame frm = new Frame(new File(ff.getId()));
                inFrames.add(frm);
                frm.setGroup(ff.isCalibFrame() ? FrameGroup.CALIB : FrameGroup.RAW);
                frm.setLevel(FrameLevel.INTERMEDIATE);
                frm.setType(FrameType.IMAGE);
                frm.setTag(ff.getClassification());
            }
        }
        return inFrames;
    }
    
    /**
     * Add input frames to the named panel. The panel must be one of the panels
     * in the list, or nothing is performed.
     * @param panelName
     * @param obComponents
     * @param queue should the components be added to the queue or the current
     * recipe panel.
     */
    public void addInputFramesToRecipePanel(String panelName, List obComponents,
            boolean queue) {
        CPLFrame panel = getRecipePanel(panelName);
        if (panel != null) {
            if (queue) {
                panel.addInputFramesToQueue(getRecipeInputFrames(obComponents));
            } else {
                panel.addInputFramesToShownRequest(getRecipeInputFrames(obComponents));
            }
            // I want the panel brought to the front, but it does not seem to work....
            panel.toFront();
        }
    }
    
    /**
     * Notify all listners of the given event.
     * @param e
     */
    private void firePropertyChange(String property, Object oldVal, Object newVal) {
        PropertyChangeEvent e = new PropertyChangeEvent(this, property,
            oldVal, newVal);
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            ((PropertyChangeListener)i.next()).propertyChange(e);
        }
    }
    
    /**
     * Add the given listener to the current listeners.
     * @param l
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        removePropertyChangeListener(l);
        listeners.add(l);
    }
    
    /**
     * Remove the given listener from the current listeners.
     * @param l
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        listeners.remove(l); 
    }
    
    /**
     * Get the requested CPLFrame, or null if not found.
     * @param name
     * @return CPLFrame
     */
    public CPLFrame getRecipePanel(String name) {
        return (CPLFrame)panels.get(name);
    }
    
    /** 
     * Get an iterator to the panel names (String objects).
     * @return Iterator
     */
    public Iterator getRecipePanelNames() {
        return panels.keySet().iterator();
    }
}
