/**
 * @author N. Kornweibel
 * @version 1.0
 *
 * The ReductionBlock as represented in Gasgano.
 */
package org.eso.gasgano.datamodel;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import org.eso.gasgano.datamodel.filesystem.DFSTextFile;
import org.eso.gasgano.properties.PropertyDB;
import org.eso.gasgano.tools.BaseName;

public class ReductionBlock extends DFSTextFile {

    // Note:
    // setting initialization values to these members
    // will over-ride the values set in the readFile()
    // method - members must be left uninitialized.
    private Vector inputFiles;
    private Vector masterCalibFiles;
    private String productBaseName;
    private String productDirectory;

    /**
     * The line in the RB which contains the product base name.
     */
    public static final int PRODUCT_LINE_NUMBER = 5;

    /**
     * After reading two lines which start with '{', we are reading
     * the files in the Master Calibration section of the RB.
     */
    public static final int NUM_LEFT_BRACES_BEFORE_CAL_FILES = 2;

    /**
     * This extensions indicate the RB failed execution.
     */
    public static final String RB_EXEC_ERR_1 = ".rZ";

    /**
     * This extensions indicate the RB failed execution.
     */
    public static final String RB_EXEC_ERR_2 = ".rY";

    /**
     * This extension indicates the RB executed without problem.
     */
    public static final String RB_EXEC_OK = ".rX";

    /**
     * This extension indicates the file has not been executed.
     */
    public static final String RB_NOT_EXEC = ".rb";

    /**
     * @param a The path to the file.
     */
    public ReductionBlock(String a) {
        super(a);
    }

    /**
     * @return A Vector of Strings of the input file names - the Vector
     * will never be null, but may be empty.
     */
    public Vector getInputFileNames() {
        return inputFiles;
    }

    /**
     * @return A Vector of Strings of the master calib file names 
     * - the Vector will never be null, but may be empty.
     */
    public Vector getMasterCalibFileNames() {
        return masterCalibFiles;
    }

    /**
     * @return The path and basename of the product listed in the RB.
     * Will never return null, but maybe an empty string.
     */
    public String getProductBaseName() {
        return productBaseName;
    }

    /**
     * @return the parent directory of the product files.
     */
    public String getProductDirectory() {
        return productDirectory;
    }

    /**
     * @return The status of the this file after parsing.
     * This method will not release data read into memory.
     * @see org.eso.gasgano.DFSFile
     */
    public int readFile() {
        int lineNo = 0;
        int braceCount = 0;
        boolean readingCalibFiles = false;
        boolean readProductFile = false;
        setStatus(FILE_VALID);
        inputFiles = new Vector();
        masterCalibFiles = new Vector();
        productBaseName = new String("");
        productDirectory = new String("");
        File theFile = new File(getAddress());
        FileReader dataSource = null;
        try {
            dataSource = new FileReader(getAddress());
        } catch (FileNotFoundException ex) {
            System.out.println("File not found: " + getAddress());
            setStatus(FILE_NOT_FOUND);
            return getStatus();
        }
        if (dataSource == null) {
            setStatus(FILE_INVALID);
            return getStatus();
        }
        StringBuffer buff = new StringBuffer();
        BufferedReader br = new BufferedReader(dataSource);
        try {
            while (true) {
                String s = br.readLine();
                lineNo++;
                if (s == null) {
                    break;
                } else {
                    if (s.startsWith("/") && readProductFile) {
                        // add it to the inputFile list.
                        int spacePlace = s.indexOf(" ");
                        String fileName = null;
                        if (spacePlace != -1) {
                            fileName = s.substring(0, spacePlace);
                        } else {
                            fileName = s;
                        }
                        if (readingCalibFiles) {
                            masterCalibFiles.addElement(fileName);
                        } else {
                            inputFiles.addElement(fileName);
                        }
                    } else if (s.startsWith("{")) {
                        braceCount++;
                        if (braceCount == NUM_LEFT_BRACES_BEFORE_CAL_FILES) {
                            readingCalibFiles = true;
                        }
                    } else if (lineNo == PRODUCT_LINE_NUMBER) {
                        // extract base name from product line.
                        File product = new File(s);
                        // extract the product base name using the specified file separator - the
                        // same one as used in the reduction block file.
                        productBaseName =
                            BaseName.extractProductBaseName(s, '/');
                        // System.out.println("From: " + s + " to: " + productBaseName);

                        productDirectory = product.getParent();
                        if (productDirectory == null
                            || productDirectory.equals("")) {
                            if (PropertyDB
                                .getInstance()
                                .isDebug(PropertyDB.debugDataModelChange)) {
                                System.err.println(
                                    "Reduction Block "
                                        + getAddress()
                                        + " is invalid, no path in product specification");
                            }

                            setStatus(FILE_INVALID);
                        }
                        readProductFile = true;
                    }
                    buff.append(s + "\n");
                }
            }
        } catch (IOException e) {
            // error reading from the Reader.
            setStatus(FILE_INVALID);
        }
        try {
            dataSource.close();
        } catch (IOException e) {
        }

        setData(buff.toString());
        return getStatus();
    } // end of method readFile().

} // end of class ReductionBlock.java

// end of file ReductionBlock.java
