/**
 * @author N. Kornweibel
 * @version 1.0
 * Collection of keyword names with column values (for reporting) which
 * can be used as a table model and read and writen to disk (ascii file).
 * 
 */

package org.eso.gasgano.gui;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.eso.gasgano.keyword.Keyword;

public class KeywordTable extends AbstractTableModel {

    private Vector[] keywords = new Vector[COLUMN_COUNT];
    private static String columnNames[] 
        = {"Keyword Name", "Report Label"};
    private String tablePath = null;
    
    /**
     * set to true when edited, and can be cleared
     * by calling resetEditFlag()
     */
    private boolean edited = false;

    private static final int COLUMN_COUNT = 2;

    public KeywordTable(String filePath) {
        tablePath = filePath;
        keywords[0] = new Vector();
        keywords[1] = new Vector();
        readFile();
        resetEditFlag();
    }
   
    public KeywordTable() {
        tablePath = null;
        keywords[0] = new Vector();
        keywords[1] = new Vector();
        insertRow(0);
        resetEditFlag();
    }

    public Vector getKeywordList() {
        return keywords[0];
    }

    public Vector getKeywordLabelList() {
        return keywords[1];
    }

    /**
     * @param c The column number.
     * @return The name of the column requirested.
     * Part of the AbstractTableModel extension.
     */
    public String getColumnName(int c) {
        return columnNames[c];
    }

    public String getTablePath() {
        return tablePath;
    }

    /**
     * @return The number of columns in the table - always 3.
     */
    public int getColumnCount() {
        return COLUMN_COUNT;
    }

    /**
     * @return The number of rows (rules) in the table.
     */
    public int getRowCount() {
        return keywords[0].size();
    }

    /**
     * @param r The row number of the cell value required.
     * @param c The column number of the cell value required.
     * @return The Object represnting the value of the cell.
     */
    public Object getValueAt(int r, int c) {
        String rc = null;
        Vector col = keywords[c];
        if (col != null) {
            try {
                rc = (String)col.elementAt(r);
            } catch (ArrayIndexOutOfBoundsException e) {
                rc = new String("");
            }
        }
        if (rc == null) {
            rc = new String("");
        }
        return rc;
    }

    /**
     * @param column The column whose width is required.
     * @return The width of the requested column.
     */
    public int getColumnWidth(int column) {
        return 30;
    }


    /**
     * @return False - the table is never editable.
     */
    public boolean isCellEditable(int r, int c) {
        return true;
    }

    /**
     * Read a KeywordTable in from disk.
     * @return True if read in without errors, else false.
     */
    public boolean readFile(String path) {
        tablePath = path;
        return readFile();
    }

    public boolean readFile() {
        if (tablePath == null) {
            return false;
        }
        boolean rc = true;
        keywords[0].removeAllElements();
        keywords[1].removeAllElements();
        try {
            BufferedReader in;
            String s = null;
            in = new BufferedReader(new FileReader(tablePath));
            while ((s = in.readLine()) != null) {
                // the first white space of the trimmed string is
                // the end of the Keyword name, the rest is the column
                // header. Invalid or null entries are ignored.
                int separator = -1;
                String keywordName = null;
                String reportName = null;
                s = s.trim();
                separator = s.indexOf(' ');
                if (separator == -1) {
                    keywordName = s;
                    reportName = new String("");
                } else {
                    try {
                        keywordName = s.substring(0, separator);
                        reportName = s.substring(separator, s.length());
                        reportName = reportName.trim();
                    } catch (StringIndexOutOfBoundsException e) {
                        keywordName = s;
                        reportName = new String("");
                    }
                }
                keywordName = keywordName.toUpperCase();
                if (Keyword.validName(keywordName)) {
                    keywords[0].addElement(keywordName);
                    keywords[1].addElement(reportName);
                } else {
                    rc = false;
                }
            }
        } catch (IOException e) {
            rc = false;
        }
        if (keywords[0].size() == 0) {
            insertRow(0);
        }
        fireTableDataChanged();
        return rc;
    }

    /**
     * Write a KeywordTable out to disk.
     * @return True if written out without problems, else false.
     */
    public boolean writeFile(String path) {
        tablePath = path;
        return writeFile();
    }

    public boolean writeFile() {
        if (tablePath == null) {
            return true;
        }
        boolean rc = true;
        try {
            FileWriter fw = new FileWriter(tablePath);
            PrintWriter pw = new PrintWriter(fw, true);
            int len = keywords[0].size();
            for (int i = 0; i < len; i++) {
                String keywordName = (String)keywords[0].elementAt(i);
                String reportName = (String)keywords[1].elementAt(i); 
                if ((keywordName == null)||(keywordName.length() == 0)) {
                    continue;
                }
                if (reportName == null) {
                    pw.println(keywordName);
                } else {
                    pw.println(keywordName+" "+reportName);
                }
            }
            pw.flush();
            pw.close();
        } catch (IOException e) {
            rc = false;
        }
        return rc;
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        edited = true;
        String val = (String)aValue;
        if (columnIndex == 0) {
            // trim, upper case, replace spaces with dots.
            val = val.trim();
            val = val.toUpperCase();
            val = val.replace(' ', '.');
        }
        Vector col = keywords[columnIndex];
        if (col != null) {
            col.setElementAt(val, rowIndex);
            fireTableDataChanged();
        }
    }

    public void removeRow(int rowNum) {
        edited = true;
        try {
            keywords[0].removeElementAt(rowNum);
            keywords[1].removeElementAt(rowNum);
            fireTableDataChanged();
        } catch (ArrayIndexOutOfBoundsException e) {
            // index was out of range...
        }
    }

    /**
     * Insert a row before the specified row number.
     */
    public void insertRow(int rowNum) {
        edited = true;
        keywords[0].insertElementAt("", rowNum);
        keywords[1].insertElementAt("", rowNum);
        fireTableDataChanged();
    }
    /**
     * Returns the edited flag, indicating if the table
     * was edited or not since the last call to resetEditFlag().
     * @return boolean
     */
    public boolean isEdited() {
        return edited;
    }
    
    /**
     * set the edit flag to false
     */
    public void resetEditFlag() {
        edited = false;
    }

} // end of class KeywordTable

// end of file KeywordTable.java
