/**
 * @author N. Kornweibel
 * @version 1.0
 * A dialog for executing a task in seperate thread, displaying its
 * progress in the dialog with a ProgressBar and Label.
 *
 * The go(), done(), getLengthOfTask(), getCurrent() and getProgressMessage()
 * methods are all abstract and require implementation.
 * The go() method should launch a thread to complete the task required
 * while the other methods should provide information on the progress
 * of the task.
 * A typical example of the implementation of go() is as follows:
 *
 *      public void go() {
 *          final SwingWorker worker = new SwingWorker() {
 *              public Object construct() {
 *                  return new BuilderThread(reclassify, rematch);
 *              }
 *          };
 *      }
 *
 * The BuilderThread class must be implemented to execute the task in
 * the constructor, updating information in which can be returned by
 * getLengthOfTask(), done(), getCurrent() and getProgressMessage() methods.
 *
 * The dialog is usually contructed, then initTask() called followed by
 * show().
 *
 */

package org.eso.gasgano.gui;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.Timer;
import javax.swing.border.EmptyBorder;

public abstract class TaskDialog extends JDialog {

    private Timer progressTimer = null;
    private JProgressBar progressBar = null;
    private JLabel progressText = null;

    public TaskDialog(Frame parent, String title, boolean modal) {
        super(parent, title, modal);
    }

    /**
     * The initTask() method is typically called before show(),
     * it starts the timer and calls go().
     */
    public void initTask() {
        progressBar = new JProgressBar(0, getLengthOfTask());
        progressBar.setValue(0);
        progressText =
            new JLabel(
                new ImageIcon(TaskDialog.class.getResource(GUIUtils.readGif)));
        progressText.setText(getProgressMessage());
        progressText.setBorder(new EmptyBorder(10, 10, 10, 10));
        progressBar.setStringPainted(true);
        getContentPane().add(progressBar, "South");
        getContentPane().add(progressText, "North");
        progressTimer = new Timer(1000, new TimerListener());
        go();
        progressTimer.start();
    }

    /**  
     * This method must launch a thread to execute the task required.
     * see header for a typical example.
     */
    public abstract void go();

    /**
     * Called from Timer to find out how much work needs
     * to be done.
     */
    public abstract int getLengthOfTask();

    /**
     * Called from Timer to find out how much has been done.
     */
    public abstract int getCurrent();

    /**
     * Called from Timer to find the progress status.
     */
    public abstract String getProgressMessage();

    /**
     * Called from Timer to find out if the task has completed.
     */
    public abstract boolean done();

    public void cleanup() {
        progressTimer.stop();
        setVisible(false);
        dispose();
    }

    /**
     * The actionPerformed method in this class
     * is called each time the Timer "goes off".
     */
    class TimerListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            if (done()) {
                cleanup();
            } else {
                progressBar.setValue(getCurrent());
                progressText.setText(getProgressMessage());
            }
        }
    }
}
