package org.eso.ohs.core.gui.widgets;

import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.SystemStatus;
import org.eso.ohs.core.utilities.SystemStatusEvent;
import org.eso.ohs.core.utilities.SystemStatusListener;

/**
 * A window for displaying/managing log messages.
 *
 * <p>
 * $Id: LogWindow.java,v 1.4 2004/04/08 12:09:51 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class LogWindow extends JFrame {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( LogWindow.class );

    /** circular buffer of displayed messages */
    private String[] msgs_ = new String[10];

    /** index into circular buffer for first message, or -1 if empty */
    private int first_ = -1;

    /** index into circular buffer for last message, or -1 if empty */
    private int last_ = -1;

    /** JTextField for displaying messages to the user. */
    private JTextArea buffer_ = new JTextArea();

    /**
     * Default constructor.
     */
    public LogWindow() {
	super();

	initialise();
    }


    /**
     * Constructor.
     * 
     * @param title	title for log window.
     */
    public LogWindow(String title) {
	super(title);

	initialise();
    }


    /**
     * Common initialiser.
     */
    private void initialise() {
	addClose();

	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(
	    new JScrollPane(buffer_),
	    BorderLayout.CENTER
	);

	setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
	setLocation(30, 30);
	setSize(700, 300);

	SystemStatus.addSystemStatusListener(new SystemStatusListener() {
	    public void systemStatusChange(SystemStatusEvent ev) {
		if (ev.getType() == SystemStatusEvent.RELOGIN) {
		    setVisible(false);
		}
	    }
	});

	buffer_.setEditable(false);
    }


    /**
     * Adds a close action to the File menu.
     */
    public void addClose() {
	String menuName = "File";
        JMenuBar menuBar = getJMenuBar();
        JMenu menu = new JMenu(menuName);
        JMenuItem mnItem;

	if (menuBar == null) {
	    menuBar = new JMenuBar();
	    setJMenuBar(menuBar);
	}

	menuBar.add(menu);
        mnItem = menu.add(new Closer(this));

        mnItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_W,InputEvent.CTRL_MASK,false)
	);

    }


    /**
     * Completely clear the contents of the Log.
     */
    public void clearMessages() {
	first_ = -1;
	last_ = -1;
	for (int j=0; j < msgs_.length; j++) {
	    msgs_[j] = null;	// help garbage collector :-)
	}

	updateBuffer();
    }


    /**
     * Add a new message to the Log.
     * An old message may be deleted, in order to make space.
     * 
     * @param line	message to be added
     */
    public void addMessage(String line) {
	if (first_ == -1) {
	    first_ = last_ = 0;
	    msgs_[first_] = line;
	}
	else {
	    last_ = (last_+1) % msgs_.length;
	    if (first_ == last_) {
			// msgs_ array was full: lose oldest message.
		first_ = (first_+1) % msgs_.length;
	    }
	    msgs_[last_] = line;
	}

	updateBuffer();
    }


    /**
     * Updates the messages visible to the user to correspond to msgs_[].
     */
    private void updateBuffer() {
	StringBuffer sb = new StringBuffer();
	if (first_ != -1) {
	    for (int j=first_; j != last_; j=(j+1)%msgs_.length) {
		sb.append(msgs_[j]);
		sb.append("\n\n");
	    }
	    sb.append(msgs_[last_]);
	}

	buffer_.setText(sb.toString());
	if (! isVisible()) {
	    setVisible(true);
	}
    }


    /**
     * Action class to close this window.
     */
    private class Closer extends AbstractAction {

	private Window wdw_;

	/**
	 * Default constructor.
	 * Sets menu item name.
	 */
	public Closer(Window wdw) {
	    super("Close");

	    wdw_ = wdw;
	}

	/**
	 * Menu item selection listener.
	 * The window is closed but not disposed of.
	 * Thus the object holding this window can re-use it.
	 */
	public void actionPerformed(ActionEvent ev) {
	    try {
		wdw_.setVisible(false);
	    }
	    catch (OutOfMemoryError e) {
		stdlog_.error(e);
		OutOfMemoryReport.showMessage();
		throw e;
	    }
	}

    }

    /**
     * Listener for "Clear" menu item.
     */
    private class ClearListener extends AbstractAction {

	/**
	 * Default constructor.
	 * Sets menu item name.
	 */
	public ClearListener() {
	    super("Clear");
	}


	/**
	 * Menu item selection listener.
	 */
	public void actionPerformed(ActionEvent ev) {
	    try {
		LogWindow.this.clearMessages();
	    }
	    catch (OutOfMemoryError e) {
		stdlog_.error(e);
		OutOfMemoryReport.showMessage();
		throw e;
	    }
	}

    }

}


