package org.eso.ohs.core.gui.widgets;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;

/**
 * Class for reporting OutOfMemoryError messages to the user.
 * This has to be special-cased because memory may be in short supply
 * when the error occurs.
 * 
 * <p>
 * To test it a utility method is included:
 * OutOfMemoryReport.useUpAllMemory().
 * This method allocates more and more data in the heap until the
 * virtual machine finally generates an OutOfMemoryError.
 * 
 * <p>
 * Typically, it will be used as in:
 * <pre>
 * 
 *     try {
 * 	    // some actions that might cause an OutOfMemoryError.
 *     }
 *     catch (OutOfMemoryError e) {
 * 	    e.printStackTrace();
 * 	    OutOfMemoryReport.showMessage();
 * 	    throw e;
 *     }
 * 
 * </pre>
 * @version $Id: OutOfMemoryReport.java,v 1.2 2004/04/06 11:50:30 tcanavan Exp $
 * 
 * <br>
 * @author Bob Kemp
 */
public class OutOfMemoryReport implements Runnable {

    private static JFrame outOfMemDialog_;

    private static OutOfMemoryReport instance_;

    private static boolean alreadyQuitting_ = false;

    static {
        initOutOfMemoryMessage();
    }

    public OutOfMemoryReport() {
        instance_ = this;
    }

    public static void showMessage() {
        System.gc();

        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension window = outOfMemDialog_.getPreferredSize();
        int x = (screen.width - window.width) / 2;
        int y = (screen.height - window.height) / 2;

        outOfMemDialog_.setLocation(x, y);
        outOfMemDialog_.pack();
        outOfMemDialog_.setVisible(true);
        if (instance_ != null && !alreadyQuitting_) {
            alreadyQuitting_ = true;
            SwingUtilities.invokeLater(instance_);
        }
    }

    /** Utility class to execute the given Runnable object afer a delay */
    public static class SlightPause extends Thread {
        Runnable runner_;

        public SlightPause(Runnable run) {
            runner_ = run;
            start();
        }
        public void run() {
            try {
                sleep(5000); // Pause 5s for display to catch up.
            } catch (InterruptedException e) {
                // do nothing -- ignore
            }
            SwingUtilities.invokeLater(runner_);
        }
    }

    public void run() {
        // If user has not clicked it away, bring window to the front.
        if (outOfMemDialog_.isVisible()) {
            outOfMemDialog_.setVisible(true);
        }

        if (instance_ != null) {
            new SlightPause(instance_);
        }
    }

    public static synchronized void initOutOfMemoryMessage() {
        outOfMemDialog_ = new JFrame("No memory");
        JButton ok = new JButton("OK");
        JButton quit = new JButton("Quit");

        outOfMemDialog_.getContentPane().setLayout(new GridLayout(0, 1));
        outOfMemDialog_.getContentPane().add(
            new JLabel(" The system does not have enough memory for that action "));
        outOfMemDialog_.getContentPane().add(
            new JLabel(" You may find it useful to quit and restart the program. "));
        outOfMemDialog_.getContentPane().add(new JLabel(""));

        Box buttons = Box.createHorizontalBox();
        buttons.add(Box.createHorizontalGlue());
        buttons.add(ok);
        buttons.add(Box.createHorizontalGlue());
        buttons.add(quit);
        buttons.add(Box.createHorizontalGlue());

        outOfMemDialog_.getContentPane().add(buttons);

        outOfMemDialog_.addNotify();

        ok.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                outOfMemDialog_.setVisible(false);
            }
        });

        quit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                System.exit(1);
            }
        });

    }

    /**
     * Forces an OutOfMemoryException by allocating an indefinitely
     * long chain of Object[] objects until the heap is exhausted.
     * This method is (of course) only used to test the handling OutOfMemoryError's.
     */
    public static void useUpAllMemory() {
        Object[] obj = null, obj2 = null;

        for (;;) {
            obj2 = obj;
            obj = new Object[10240];
            obj[0] = obj2;
        }
    }

    /** To test this class: all it does is call the useUpAllMemory()
     *  method. 
     *
     * <br>ChangeLog: A M Chavan, 02-Feb-01
     */
    public static void main(String[] args) {
        useUpAllMemory();
    }

}
