/* Created on Apr 25, 2004 */

package org.eso.ohs.core.utilities;

/**
 * A specialized kind of Range, specific for ESOFORM proposals:
 * lower limit is never negative, can be created from a descriptive
 * String.
 * 
 * @author amchavan
 * @version $Revision: 1.1 $
 */

// $Id: Cardinality.java,v 1.1 2004/04/26 15:52:42 amchavan Exp $

public class Cardinality extends Range {

    /** Default constructor: build a range with
     *  lower limit equal to zero and Integer.MAX_VALUE as upper limit
     */
    public Cardinality() {
        super( 0, Integer.MAX_VALUE );
        absLower = 0;
    }

    /**
     * Construct a range with the given lower and upper limits
     * @throws RuntimeException if upper smaller than lower OR lower is negative
     */
    public Cardinality( int lower, int upper ) {
        super( lower, upper );
        if( lower < 0 )
            throw new RuntimeException( "negative lower limit" );
    }

    /** Construct a Cardinality from a description String. 
     * Depending on the input string, the range includes:
     * <UL>
     * <LI> <code>null</code>, or the empty string: 
     *      any value within the default limits.
     * <LI> <em>integer</em>: only that integer
     * <LI> <em>lower</em><code>..</code><em>upper</em>: 
     *      any value between <em>lower</em> and <em>upper</em>, 
     *      including the limits. <em>lower</em> and <em>upper</em>
     *      must be integer numbers; for instance <code>0..9</code>
     * <LI> <em>lower</em><code>..</code>: 
     *      any value greater than or equal to <em>lower</em>. <em>lower</em>
     *      must be an integer number; for instance <code>1..</code>
     * <LI> <code>..</code><em>upper</em>: 
     *      any value smaller than or equal to <em>upper</em>. <em>upper</em>
     *      must be an integer number; for instance <code>..10</code>
     * <LI> <code>*</code>: any value greater than or equal to zero.
     *      That is equivalent to <code>0..</code>
     * <LI> <code>+</code>: any value greater than or equal to one.
     *      That is equivalent to <code>1..</code>
     * </UL>
     * 
     * @throws RuntimeException if the input String cannot be recognized
     *                          or the resulting range is invalid.
     */
    public Cardinality( String range ) {
        this();			// set the default limits
        
        int l;			// length of the input string
        
        // If input string is null or empty, keep the
        // default uppe and lower limit
        if( range == null || (l = range.length()) == 0 )
            return;
        
        // "*" --> any non-negative number
        // "+" --> any strictly positive number
        if( l == 1 ) {
            char c = range.charAt(0);
    
            if( c == '*' ) {
                lower = 0;
                return;
            }
            
            if( c == '+' ) {
                lower = 1;
                return;
            }
        }
        
        // An integer translates to a range containing
        // only itself
        try {
            upper = Integer.parseInt( range );
            lower = upper;
            return;
        }
        catch( NumberFormatException e ) {
            // not an integer, let's go on...
        }
        
        // Let's see if the input string is something like "<int1>..<int2>".
        // (could use a pattern here...)
        int s = range.indexOf( SEPARATOR );
        if( s != -1 ) {
            
            try {
                String int1 = range.substring( 0, s );		// this is <int1>
                int l1 = int1.length();
                if(  l1 > 0 )
                    lower = Integer.parseInt( int1 );
                else 
                    lower = Integer.MIN_VALUE;
                
                String int2 = range.substring( s + 2 );		// this is <int2>
                int l2 = int2.length();
                if( l2 > 0 )
    	            upper = Integer.parseInt( int2 );
                else
                    upper = Integer.MAX_VALUE;
                
                if( l1 == 0 && l2 == 0 )
                    throw new RuntimeException( "unrecognized input: '" + 
                                                range + "'" );
                    
                checkLowerUpper();					
                return;
            }
            catch( NumberFormatException e ) {
                // not an "<int1>..<int2>" string, let's go on...
            }
        }
        
        // We give up!
        throw new RuntimeException( "unrecognized input: '" + range + "'" );
    }
}
