package org.eso.ohs.core.utilities.rmi;

import java.rmi.ConnectException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.UnmarshalException;

import org.apache.log4j.Logger;




/**
 * This class defines a client for an OHSServer (or derived class)
 * which controls the server via its OHSServerControl interface.
 *
 * @version $Revision: 1.3 $
 *
 * @see #main 
 * @author amchavan, 15-Oct-2001: Creation
 */
public class OHSServerController {

    public static final String rcsid = "$Id: OHSServerController.java,v 1.3 2004/04/08 12:09:51 tcanavan Exp $";

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( OHSServerController.class );


    /** Reference to the remote server we want to control */
    OHSServerControl server_ = null;

    /** URL of the remote server we want to control */
    String url_ = null;

    /**
     * In case of any error the 
     * @param oshServer URL of the OHS server to control
     * @throws NotBoundException If the server cannot be contacted.
     */
    OHSServerController( String ohsServer ) throws Exception {

	url_ = ohsServer;
	server_ = (OHSServerControl) Naming.lookup( ohsServer );
    }

    
    /** 
     * Terminate the server
     * @param xstat     Exit status of the server.
     */
    public void terminate( int xstat ) throws Exception {
	try {
	    server_.exit( xstat );
	    stdlog_.debug( url_ + ": terminated with status " + xstat );
	}
	catch( UnmarshalException e ) {
	    // When a server terminates, the client may receive
	    // an UnmarshalException -- it appears to be OK,
	    // so we don't do anything about it.
	}
    }

    /** 
     * Ping the server.
     */
    public void ping() throws Exception {
	try {
	    server_.ping();
	}
	catch( Exception e ) {
	    // When a server terminates, the client may receive
	    // an UnmarshalException -- it appears to be OK,
	    // so we don't do anything about it.
	}
    }

    /** 
     * Print a list of the all servers bound to the RMI registry.
     * @param url   URL of the registry to be contacted (the
     *              <em>file</em> component of the URL is ignored).
     *
     * @see java.rmi.Naming#list
     */
    public static void list ( String url ) throws Exception {
	String[] l =  Naming.list( url );
	for( int i = 0; i < l.length; i++ ) 
	    stdlog_.debug( l[i] );
    }

    /**
     * Usage: <code>java org.eso.ohs.cliserv.OHSServerController</code>
     *        <em>url</em> <em>command</em> [<em>args</em>...]<BR>
     * where <em>command</em> can be:<UL>
     * <li> <code>list</code>: print the list of all servers bound to 
     *      the RMI registry; the registry is contacted using the
     *      <em>url</em> arg  (the <em>file</em> component of the URL 
     *      is ignored).
     * <li> <code>ping</code>: ping the server.
     * <li> <code>terminate</code>: terminate the server; in this
     *       case the next arg is the exit status of the server.
     * </ul>
     *
     * For instance:<BR>
     * <code>
     * &nbsp;&nbsp;java org.eso.ohs.cliserv.OHSServerController&nbsp;rmi://usg0.hq.eso.org/P2PP-2&nbsp;1
     * </code>
     * <P>
     *
     * Upon successful completion of a command the controller exits
     * with a zero status.
     * <BR>
     * If the server or the RMI registry cannot be contacted
     * the exit status is 2.
     * <BR>
     * In the case of an error the controller exits with 1.
     * <P>
     *
     * Info messages are produced on <em>stdout</em>.
     *
     * <bf>Note</bf>: Command line args are <em>not</em> checked.
     */
    public static void main( String[] args ) {
	
	OHSServerController ctl = null;
	String              url = args[0];
	String              cmd = args[1];

	try {
	    // Switch on the command arg
	    //----------------------------------------
	    if( cmd.equals( "list" )) {
		list( url );
	    } 
	    else if( cmd.equals( "ping" )) {

		ctl = new OHSServerController( url );
		ctl.ping();
	    } 
	    else if( cmd.equals( "terminate" )) {

		int status = (new Integer( args[2] )).intValue();
		ctl = new OHSServerController( url );		
		ctl.terminate( status );
	    }
	    else {
		stdlog_.debug( cmd + ": unknown command." );
		System.exit( 1 );
	    }
	}

	catch( ConnectException e ) {
	    stdlog_.error(  "Registry: cannot contact: " + 
				 e.getMessage() );
	    System.exit( 2 );
	}
	catch( NotBoundException e ) {
	    stdlog_.error( url + ": cannot contact." );
	    System.exit( 2 );
	}
	catch( Exception e ) {
	    stdlog_.error( url + ": " + 
				cmd + ": exception: " + e.getMessage() );
	    e.printStackTrace();
	    System.exit( 1 );
	}
    }
}

