package org.eso.ohs.phase2.apps.p2pp;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Enumeration;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.event.EventListenerList;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryEvent;
import org.eso.ohs.persistence.DirectoryListener;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectEvent;
import org.eso.ohs.persistence.ObjectListener;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * A GUI class for displaying the hierarchy of folders on a given medium, using
 * a JTree. The user can manipulate the folders and display their contents.
 * 
 * <p>
 * $Id: HierarchyView.java,v 1.11 2005/09/09 07:56:27 tcanavan Exp $
 */

public class HierarchyView extends JPanel {

    private static Logger stdlog_ = Logger.getLogger(HierarchyView.class);

    /**
     * Possible argument to setDisplayedPeriod() to indicate that all periods
     * should be displayed.
     */
    public static final int ALL_PERIODS = -1;

    /** GUI component used to display the tree of folders. */
    private JTree tree_;

    /** medium on which the folders exist. */
    private Media device_;

    /** Model for the JTree */
    private FolderTreeModel treeModel_;
    
    
    private FolderGridView folderGridView_;

    /**
     * Constructor for the HierarchyView class.
     * 
     * @param device
     *            medium containing the folders to be displayed.
     */
    HierarchyView(Media device, DirectoryNode root,FolderGridView folderGridView) {
        device_ = device;
        folderGridView_ = folderGridView;
        tree_ = new JTree();
        // remove Ctrl-A key action -- we define our own in the menus
        // tree_.getInputMap().remove(KeyStroke.getKeyStroke(KeyEvent.VK_A,InputEvent.CTRL_MASK));
        tree_.getActionMap().put("selectAll", new SelectAllAction());
        // tree_.unregisterKeyboardAction(KeyStroke.getKeyStroke(KeyEvent.VK_A,InputEvent.CTRL_MASK));
        treeModel_ = new FolderTreeModel(device, root);
        tree_.setModel(treeModel_);

        ToolTipManager.sharedInstance().registerComponent(tree_);
        setRoot(root);
        TreeSelectionModel tsm = tree_.getSelectionModel();
        tsm.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        tree_.setSelectionRow(0);
        tree_.setInvokesStopCellEditing(true);
        tree_.setShowsRootHandles(true);
        tree_.setCellRenderer(new HierTreeRenderer());
        setRootVisible(false);

        // Configure default renderer to display a "leaf" the
        // same as a "node", ie so that folders appear the same
        // regardless of whether they contain other folders or not.
        DefaultTreeCellRenderer rend;
        rend = (DefaultTreeCellRenderer) tree_.getCellRenderer();
        rend.setLeafIcon(rend.getOpenIcon());

        setLayout(new BorderLayout());
        add(new JScrollPane(tree_), BorderLayout.CENTER);
        setBorder(BorderFactory.createRaisedBevelBorder());
    }

    public void setDisplayedPeriod(int period) {
        int oldPeriod = getDisplayedPeriod();

        treeModel_.setDisplayedPeriod(period);
        if (period != oldPeriod) {
            selectInitialFolder();
        }
    }

    public int getDisplayedPeriod() {
        return treeModel_.getDisplayedPeriod();
    }

    /**
     * Adds a new listener to be notified of changes in the selection.
     * 
     * @param lnr
     *            the listener
     */
    public void addSelectionListener(TreeSelectionListener lnr) {
        tree_.addTreeSelectionListener(lnr);
    }

    /**
     * Removes a listener, which will no longer be notified of changes.
     * 
     * @param lnr
     *            the listener
     */
    public void removeSelectionListener(TreeSelectionListener lnr) {
        tree_.removeTreeSelectionListener(lnr);
    }

    /**
     * Removes a listener, which will no longer be notified of changes.
     * 
     * @param lnr
     *            the listener
     */
    public void selectInitialFolder() {
        tree_.clearSelection();
        tree_.setSelectionInterval(0, 0);
    }

    /**
     * Make the JTree root visible or invisible.
     * 
     * @param visible
     *            true if the root should be visible, false otherwise.
     */
    public void setRootVisible(boolean visible) {
        tree_.setRootVisible(visible);
    }

    /**
     * Returns the currently selected folder in the hierarchy.
     * 
     * @return currently selected folder.
     * 
     * @exception ObjectIOException
     *                if an I/O error occurred reading/creating the folder
     */
    public Folder getSelectedFolder() throws ObjectIOException {
        // Java1.4 port TreePath path = tree_.getLeadSelectionPath();
        TreePath path = tree_.getSelectionPath();
        DefaultMutableTreeNode node;
        HierarchyView.DirInfo info;

        if (path == null) {
            return null; // nothing selected
        }

        node = (DefaultMutableTreeNode) path.getLastPathComponent();
        info = (HierarchyView.DirInfo) node.getUserObject();

        return FolderFactory.getFolder(device_, info.node);
    }

    /**
     * Selects the specified folder in the folder hierarchy.
     * 
     * @param folder
     *            folder to be selected
     */
    public void setSelectedFolder(Folder folder) {
        DefaultMutableTreeNode tnode = treeModel_.findTreeNodeforDirNode(folder
                .getNode());

        tree_.setSelectionPath(new TreePath(tnode.getPath()));
    }

    /**
     * Returns the root node of the displayed folder hierarchy.
     * 
     * @return DirectoryNode specifying the root node.
     */
    public DirectoryNode getRoot() {
        return (DirectoryNode) treeModel_.getRoot();
    }

    /**
     * Sets the root node of the displayed folder hierarchy.
     * 
     * @param root
     *            DirectoryNode specifying the new root node.
     */
    public void setRoot(DirectoryNode root) {
        // The root of a hierarchy must be the root itself
        // or an immediate child of the root.
        if (root.getParent() != null) {
            while (root.getParent().getParent() != null) {
                root = root.getParent();
            }
        }

        treeModel_.setRoot(new DefaultMutableTreeNode(new DirInfo(root)));

        tree_.revalidate();
        tree_.repaint(40);
    }

    /**
     * Sorts the given array of DirectoryNode objects with names corresponding
     * to observing runs. The ordering is one that is suitable for observing
     * runs and effectively sorts them by program id but ignores the leading
     * space or "1" that indicates program type.
     * 
     * @param subdirs
     *            array of subdirectories to be sorted in-place.
     */
    public static DirectoryNode[] sortDirNodes(DirectoryNode[] subdirs) {
        DirectoryNode tmp;
        boolean topLevelDirs;

        if (subdirs == null || subdirs.length == 0) {
            return new DirectoryNode[] {};
        }

        topLevelDirs = (subdirs[0].getParent() == null);

        for (int j = 0; j < subdirs.length; j++) {
            String descrJ = subdirs[j].getDescription();

            // For top-level dirs, drop first char of ObsRun name
            if (topLevelDirs)
                descrJ = descrJ.substring(1);

            for (int k = j + 1; k < subdirs.length; k++) {
                String descrK = subdirs[k].getDescription();

                // For top-level dirs, drop first char of ObsRun name
                if (topLevelDirs)
                    descrK = descrK.substring(1);

                if (descrJ.compareTo(descrK) > 0) {
                    tmp = subdirs[j];
                    subdirs[j] = subdirs[k];
                    subdirs[k] = tmp;

                    descrJ = descrK;
                }
            }
        }

        return subdirs;
    }

    public class FolderTreeModel extends DefaultTreeModel implements
            DirectoryListener, ObjectListener, PropertyChangeListener {
        private DirectoryNode root_;

        private Media device_;

        /** Period for observing runs to dispaly to the user */
        private int displayedPeriod_ = ALL_PERIODS;

        /** Root of the tree of JTree TreeNode objects */
        private DefaultMutableTreeNode rootNode_;

        /* Support for handling directory events */
        protected EventListenerList treeListeners_ = new EventListenerList();

        public FolderTreeModel(Media device, DirectoryNode root) {
            super(new DefaultMutableTreeNode(new DirInfo(root)));

            device_ = device;
            root_ = root;

            rootNode_ = (DefaultMutableTreeNode) this.getRoot();
            updateHierarchy();

            Persistence objMgr = Persistence.getInstance();
            objMgr.addDirectoryListener(this);
            objMgr.addObjectListener(this);
        }

        public void setRoot(DefaultMutableTreeNode newRoot) {
            super.setRoot(newRoot);

            rootNode_ = newRoot;
            root_ = ((DirInfo) rootNode_.getUserObject()).node;

            updateHierarchy();
        }

        public void setDisplayedPeriod(int period) {
            displayedPeriod_ = period;
            rootNode_.removeAllChildren();
            updateHierarchy();
            reload();
        }

        public int getDisplayedPeriod() {
            return displayedPeriod_;
        }

        public void objectAdded(ObjectEvent ev) {

            if (ev.getObjectClass() == ObservingRun.class) {
                try {
                    Persistence objMgr = Persistence.getInstance();
                    final DirectoryNode dir = ev.getDirectoryNode();
                    ObservingRun or = (ObservingRun) objMgr.getBusObj(ev
                            .getMedia(), ev.getObjectId(), ev.getObjectClass());
                    or.addPropertyChangeListener(this);
                    FolderFactory.getFolder(ev.getMedia(), dir);

                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            TreeNode node = findTreeNodeforDirNode(dir);
                            reload(node);
                        }
                    });

                } catch (ObjectNotFoundException e) {
                    stdlog_.error(e);
                } catch (ObjectIOException e) {
                    stdlog_.error(e);
                }
            }
        }

        public void objectMoved(ObjectEvent ev) {
            // ignore for ObsRuns
        }

        public void objectRemoved(ObjectEvent ev) {
            // ignore for ObsRuns
        }

        public void objectRemovedFromRegistry(ObjectEvent ev) {
            // ignore for ObsRuns
        }

        public void propertyChange(PropertyChangeEvent ev) {
            ObservingRun or = (ObservingRun) ev.getSource();
            Persistence objMgr = Persistence.getInstance();
            DirectoryNode dir = objMgr.getDirectoryFor(or);
            DefaultMutableTreeNode node = findTreeNodeforDirNode(dir);

            nodeChanged(node);
        }

        /**
         * A directory has been added.
         * 
         * @param ev
         *            DirectoryEvent with more specific information.
         */
        public void directoryAdded(DirectoryEvent ev) {
            DirectoryNode node = ev.getDirectoryNode();
            DirectoryNode parent = node.getParent();

            insertNodeBelow(parent, node);
            updateHierarchy();
        }

        private void insertNodeBelow(DirectoryNode parent, DirectoryNode node) {
            DefaultMutableTreeNode treeParent = findTreeNodeforDirNode(parent);
            DirInfo di = new DirInfo(node);
            DefaultMutableTreeNode treeChild = new DefaultMutableTreeNode(di);
            int currChildCount = treeParent.getChildCount();
            String newName = di.toString();

            for (int j = 0; j < currChildCount; j++) {
                DefaultMutableTreeNode childJ = (DefaultMutableTreeNode) treeParent
                        .getChildAt(j);
                String nameJ = childJ.getUserObject().toString();

                if (nameJ.compareTo(newName) > 0) {
                    insertNodeInto(treeChild, treeParent, j);
                    updateHierarchy(node, treeChild);
                    return;
                }
            }

            insertNodeInto(treeChild, treeParent, currChildCount);
            updateHierarchy(node, treeChild);
        }

        /**
         * A directory has been moved.
         * 
         * @param ev
         *            DirectoryEvent with more specific information.
         */
        public void directoryMoved(DirectoryEvent ev) {
            DirectoryNode node = ev.getDirectoryNode();
            DirectoryNode parent = node.getParent();
            DefaultMutableTreeNode treeNode = findTreeNodeforDirNode(node);

            if (treeNode == null) {
                return;
            }

            removeNodeFromParent(treeNode);

            insertNodeBelow(parent, node);
            // expandTree(findTreeNodeforDirNode(parent));
        }

        /**
         * A directory has been changed.
         * 
         * @param ev
         *            DirectoryEvent with more specific information.
         */
        public void directoryChanged(DirectoryEvent ev) {
            DirectoryNode node = ev.getDirectoryNode();
            DirectoryNode parent = node.getParent();
            DefaultMutableTreeNode treeNode = findTreeNodeforDirNode(node);

            removeNodeFromParent(treeNode);

            insertNodeBelow(parent, node);
            // expandTree(findTreeNodeforDirNode(parent));
        }

        /**
         * A directory has been removed.
         * 
         * @param ev
         *            DirectoryEvent with more specific information.
         */
        public void directoryRemoved(DirectoryEvent ev) {
            DirectoryNode node = ev.getDirectoryNode();

            try {
                Folder folder = FolderFactory.getFolder(device_, node
                        .getParent());
                setSelectedFolder(folder);
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(null, e);
            }

            removeNodeFromParent(findTreeNodeforDirNode(node));
        }

        /**
         * Recursively updates the JTree's TreeNode hierarchy that models the
         * hierarchy of folders.
         */
        public void updateHierarchy() {
            updateHierarchy(root_, rootNode_);
            expandTree(rootNode_);
        }

        public void expandTree(DefaultMutableTreeNode node) {
            if (tree_ != null) {
                // Force all nodes of the tree to be expanded.
                // This call should go _after_ all nodes added,
                // otherwise node labels are shown as ellipses (...)
                // XXX Find out why the order is so important
                Enumeration enum = node.depthFirstEnumeration();
                DefaultMutableTreeNode treeNode = null;

                while (enum.hasMoreElements()) {
                    treeNode = (DefaultMutableTreeNode) enum.nextElement();
                    tree_.expandPath(new TreePath(treeNode.getPath()));
                }
            }
        }

        /**
         * Recursively updates a subtree of the JTree's TreeNode hierarchy that
         * models the hierarchy of folders. The input data comes from the
         * DirectoryNode argument.
         * 
         * @param dir
         *            subtree of DirectoryNodes.
         * @param node
         *            subtree of TreeNode to be updated.
         */
        private void updateHierarchy(DirectoryNode dir,
                DefaultMutableTreeNode node) {
            DirectoryNode[] subdirs = dir.getDirectories();
            DefaultMutableTreeNode treeChild;
            DirectoryNode nodeChild;
            DirInfo di;

            // Top level (Observing Run) directories?
            if (dir.getParent() == null) {
                int period = getDisplayedPeriod();

                if (period != ALL_PERIODS) {
                    subdirs = Phase1TargetsViewer.selectObsRunDirs(subdirs,
                            period);
                }
            }

            subdirs = sortDirNodes(subdirs);

            addANameLoop: for (int j = 0; j < subdirs.length; j++) {
                nodeChild = subdirs[j];

                for (int k = node.getChildCount() - 1; k >= 0; k--) {
                    treeChild = (DefaultMutableTreeNode) getChild(node, k);
                    di = (DirInfo) treeChild.getUserObject();
                    if (nodeChild == di.node) {
                        // Already in the tree -- recurse over children
                        updateHierarchy(nodeChild, treeChild);
                        continue addANameLoop;
                    }
                }

                di = new DirInfo(nodeChild);

                treeChild = new DefaultMutableTreeNode(di);
                insertNodeInto(treeChild, node, j);
                updateHierarchy(nodeChild, treeChild);
            }

        }

        /**
         * Finds and returns the JTree node for a given DirectoryNode, or null
         * if not found.
         * 
         * @param dirNode
         *            path in hierarchy for which node is sought
         */
        public DefaultMutableTreeNode findTreeNodeforDirNode(
                DirectoryNode dirNode) {
            Enumeration enum = rootNode_.depthFirstEnumeration();
            DefaultMutableTreeNode treeNode = null;

            while (enum.hasMoreElements()) {
                treeNode = (DefaultMutableTreeNode) enum.nextElement();
                if (((DirInfo) treeNode.getUserObject()).node == dirNode) {
                    return treeNode;
                }
            }

            return null;
        }

    }

    /**
     * Utility class to hold per-JTree node information. The main reason for
     * this class is so that the toString() method of the TreeNode user objects
     * actually calls the getDescription() method.
     */
    public static class DirInfo {
        public DirectoryNode node;

        public DirInfo(DirectoryNode node) {
            this.node = node;
        }

        public String toString() {
            return node.getDescription();
        }
    }

    private class HierTreeRenderer extends DefaultTreeCellRenderer {
        public Component getTreeCellRendererComponent(JTree tree, Object value,
                boolean selected, boolean expanded, boolean leaf, int row,
                boolean hasFocus) {

            Component c = super.getTreeCellRendererComponent(tree, value,
                    selected, expanded, leaf, row, hasFocus);
            if (row >= 0) {
                TreePath treePath = tree.getPathForRow(row);

                if (treePath == null) {
                    return c;
                }

                DefaultMutableTreeNode mutNode = (DefaultMutableTreeNode) treePath
                        .getLastPathComponent();
                DirInfo info = (DirInfo) mutNode.getUserObject();
                DirectoryNode node = info.node;
                Folder folder = (Folder) node.getObject();
                ObservingRun obsRun = folder.findObsRun();
                String title = null;
                if (obsRun != null) {
                    title = "Programme Title : " + obsRun.getTitle();
                }
                if (title == null || title.length() <= 0)
                    title = "No title available";
                ((JComponent) c).setToolTipText(title);
            }
            return c;
        }
    }

    private class SelectAllAction extends AbstractAction {


        SelectAllAction() {

        }

        public void actionPerformed(ActionEvent ae) {
             folderGridView_.selectAllObjects();
        }
    }

}
