package org.eso.ohs.phase2.visibility;

/**
 * Moon Distance class calculates the Moon angular 
 * separation distance the following SLALIB routines are called:
 *
 * slaRdplan - Moon position
 * slaDsep - angular separation of the moon and target
 *
 * @author C. Mavros for ESO
 * @version 25 July 2001
 */

import org.apache.log4j.Logger;

public class MoonDistance extends JNIWrapper {

    /**
     * Standard log channel
     * <br>ChangeLog: cmavros 21-12-01 added
     */
    private static Logger stdlog_ = Logger.getLogger(MoonDistance.class);

    /**
    * The constant the SLALIB function uses
    * for the Moon
    */
    private static final int MOON = 3;

    /**
    * Position in RA and DEC position in the
    * array returned from the SLALIB function
    */
    protected static final int MOON_RA = 0;
    protected static final int MOON_DEC = 1;

    /**
    * 180/PI  Convert radians to degrees 
    */
    private static final double R2D = 57.2957795130823;

    /**
     * pi/12:  hours to radians 
     */
    private static final double H2R = 0.2617993877992;

    /**
    * getMoonDistance calculates the Moon's angular 
    * separation distance. By calling slaRdplan for the Moon 
    * and then calling slaDsep for the target. Giving you how 
    * far apart the target and Moon are in the sky.
    *
    * Note: Included is an another calculation for moon distance
    * taken for the Skycalc program to use as a check
    * if neccessary.
    *
    * @param jDate Julian Date of OB 
    * @param ra of target 
    * @param dec of target 
    * @param longitude of observatory 
    * @param lat of observatory 
    * @param boolean considerHorizon if true and moon is below the horizon
    *     return -1.0 instead of the moon distance. 
    * @return the Moon angular separation distance in degrees 
    */

    public static double getMoonDistance(
        double jDate,
        double ra,
        double dec,
        double longitude,
        double lat) {

        double rdpMoon[] = new double[3];

        rdpMoon = getMoonCoord(jDate, longitude, lat);

        //stdlog_.info ("visiplot.MoonDistance.getMoonDistance moonRA "+ rdpMoon[MOON_RA]/H2R);
        //stdlog_.info ("visiplot.MoonDistance.getMoonDistance moonDEC "+ rdpMoon[MOON_DEC]*R2D);

        double rc;

        /** Pass the previously calculated RA and Dec of the Moon
             * to the SLALIB function slaDsep to
             * calculate the distance between the Moon and the target
             */
        rc = jnislaDsep(rdpMoon[MOON_RA], rdpMoon[MOON_DEC], ra, dec);

        /* Sanity check to calculate the angle between the Moon and the target  
           double checkResult = subtend(rdpMoon[MOON_RA], rdpMoon[MOON_DEC], ra, dec);
           stdlog_.info ("visiplot.MoonDistance.getMoonDistance = "+ checkResult*R2D);
        */
        rc = rc * R2D;
        return (rc);

    }

    /**
     * Determine if the moon is visible or not.
     * @param jDate
     * @param longitude
     * @param lat
     * @return double 1.0 if the moon is above the horizon, else 0.0
     */
    public static double getMoonVisible(
        double jDate,
        double longitude,
        double lat) {

        double rdpMoon[] = new double[3];

        rdpMoon = getMoonCoord(jDate, longitude, lat);
        boolean moonVisible =
            getMoonVisible(
                jDate,
                rdpMoon[MOON_RA],
                rdpMoon[MOON_DEC],
                longitude,
                lat);
        // stdlog_.info("visiplot.MoonDistance.getMoonVisible()? "+ moonVisible);
        return moonVisible ? 1.0 : 0.0;
    }

    /**
	 * @param jDate
	 * @param longitude
	 * @param lat
	 * @return
	 */
	protected static double[] getMoonCoord(double jDate, double longitude, double lat) {
		double[] rdpMoon;
		/** 
         * Call SLALIB function slaRdplan for the Moon position.
         * Returning the RA and Dec of the moon in radians.
         */
        rdpMoon = jnislaRdplan((jDate - 2400000.5), MOON, longitude, lat);
		return rdpMoon;
	}

	/**
     * @param julian date
     * @param moonRa
     * @param moonDec
     * @param longitude of observatory
     * @param lat of observatory  (radians)
     * @param lst (radians)
     * @return true if the moon is below the horizon 
     */
    private static boolean getMoonVisible(
        double jDate,
        double moonRa,
        double moonDec,
        double longitude,
        double lat) {

        double result = getMoonAngle(jDate, moonRa, moonDec, longitude, lat);

        return (result >= 0.0);
    }

    /**
	 * @param jDate
	 * @param moonRa
	 * @param moonDec
	 * @param longitude
	 * @param lat
	 * @return
	 */
	protected static double getMoonAngle(double jDate, double moonRa, double moonDec, double longitude, double lat) {
		double rdpDe2h[] = new double[2];
        double lst = SiderealTime.getST(jDate) - longitude;
        /** 
         * Hour Angle - The angle between an observer's meridian and the 
         * hour circle on which some celestial body lies.
         */
        double HA = lst - moonRa;

        /**
         * Moon twilight angle in radians
         */
        rdpDe2h = jnislaDe2h(HA, moonDec, lat);
        double result = rdpDe2h[1] * R2D;
		return result;
	}

	/**
    * Sanity check method to calculate the 
    * angle subtended by two positions in the sky 
    *
    * @param ra and dec of the first object 
    * @param ra and dec of the second object 
    * @return value is in radians
    */
    private static double subtend(
        double ra1,
        double dec1,
        double ra2,
        double dec2) {

        double x1 = Math.cos(ra1) * Math.cos(dec1);
        double y1 = Math.sin(ra1) * Math.cos(dec1);
        double z1 = Math.sin(dec1);
        double x2 = Math.cos(ra2) * Math.cos(dec2);
        double y2 = Math.sin(ra2) * Math.cos(dec2);
        double z2 = Math.sin(dec2);
        double theta = Math.acos(x1 * x2 + y1 * y2 + z1 * z2);

        /* Use flat Pythagorean approximation if the angle is very small
         * and you're not close to the pole; avoids roundoff in arccos. 
         */
        if (theta < 1.0e-5) { /* seldom the case, so don't combine test */
            if (Math.abs(dec1) < (Math.PI / 2 - 0.001)
                && Math.abs(dec2) < (Math.PI / 2 - 0.001)) {
                x1 = (ra2 - ra1) * Math.cos((dec1 + dec2) / 2.);
                x2 = dec2 - dec1;
                theta = Math.sqrt(x1 * x1 + x2 * x2);
            }
        }
        return (theta);
    }
}
