package org.eso.ohs.scripting;

/**
 * Implementation of the ScriptingInterpreter interface for Python. Using the
 * JPython script interpreter.
 *
 * The static initializer will register a singleton instance of this class
 * on application startup.
 *
 * An instance to this object should only be obtained through the
 * ScriptInterpreterManager object.
 *
 * @author Nick Kornweibel
 */

import java.io.File;

import org.python.core.PyException;
import org.python.util.PythonInterpreter;

public class JPythonScriptInterpreter implements ScriptInterpreter {

    /** singleton instance of the JaclScriptInterpreter */
    private static JPythonScriptInterpreter instance;

    /** languages supported */
    private static final String[] langs = {"Python"};

    /** file extensions supported */
    private static final String[] exts = {"py"};

    /** single instance of the JPython interpreter */
    private static PythonInterpreter interpreter = null;

    /**
     * Static initializer that is executed when the class is initialized
     */

    static {
        if (getInterpreter() != null) {
            instance = new JPythonScriptInterpreter();
            ScriptInterpreterManager.getInstance()
                     .registerScriptInterpreter(instance);
        }
    }

    /**
     * Private default constructor 
     */

    private JPythonScriptInterpreter() {}

    private static PythonInterpreter getInterpreter() {
        if (interpreter == null) {
            try {
                Class.forName("org.python.util.PythonInterpreter");
                interpreter = new PythonInterpreter();
            } catch (ClassNotFoundException ex) {
                interpreter = null;
            }
        }
        return interpreter;
    }

    /**
     * Execute the script text passed
     * @param script The text to be executed
     */

    public void executeScript(String script)
            throws ScriptInterpreterException {
        try {
            getInterpreter().exec(script);
        } catch (PyException ex) {
            throw (new ScriptInterpreterException(ex.toString()));
        }
    }

    /**
     * Execute the text in the file given
     * @param scriptPath The path to the text file
     */

    public void executeFile(File scriptPath)
            throws ScriptInterpreterException {
        try {
            getInterpreter().execfile(scriptPath.getPath());
        } catch (PyException ex) {
            throw (new ScriptInterpreterException(ex.toString()));
        }
    }


    /**
     * Get the names of the languages supported by this interpreter
     * @return Array of String objects of the supported language names
     */

    public String[] getLanguageNames() {
        return langs;
    }

    /**
     * Get the file extensions supported by this interpreter
     * @return Array of String objects of the supported file extensions
     */

    public String[] getFileExtensions() {
        return exts;
    }
}
