package org.eso.ohs.scripting;

/**
 * To encapsulate the interfaces exposed to the scripting environment, 
 * (and to build an illusion to the script developer that they their scripts 
 * are executed in a controlled environment), the ScriptingBox was designed 
 * is the primary interface used by the script developer.
 * 
 * It is through this interface that the OB is obtained, info, warning 
 * and error messages are reported.
 * 
 * The ScriptingBox is an abstract base class, requiring implementation of:
 *     getObservationBlocks()
 * 
 * The intended entry point for scripts accessing the OB data hierarchy is 
 * with a call to: ScriptingBox.getInstance(), which will return the 
 * current ScriptingBox (implemented by extending class).
 * When subclassing, ensure the default constructor for this class is
 * called to set the static instance member.
 *
 * @author Nick Kornweibel
 */
 
import org.apache.log4j.Logger;

public abstract class ScriptingBox {
    
    /** ScriptingUtilities instance */
    protected ScriptingUtilities scriptingUtils;
    
   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger(ScriptingBox.class);


    private static ScriptingBox instance_;

    /**
     * Default constructor - must be called in order to set the
     * static instance that is returned to the scrinting environment
     * via ScriptingBox.getInstance()
     */

    public ScriptingBox() {
        instance_ = this;
    }

    /**
     * Get an instance of the ScriptingBox
     * @return instance of SciptingBox to be used in the scripting environment
     */

    public static ScriptingBox getInstance() {
        if(instance_ == null)
        System.out.println("!!!!!!!!!!!!!!!!! Imoending Disaster");
        return instance_;
    }

    /** Version String - see getVersion() */

    private static final String version = "OBData_ScriptingBox_1_1";

    /**
     * All classes and interfaces contained in the ScriptingBox or 
     * returned by the ScriptingBox are under revision control, and 
     * any changes made to these interfaces or classes must be reflected 
     * by a change in the version string member of the ScriptingBox. 
     * The version string allows for the run-time checking of the 
     * scripting interface by the script module (as interface incompatibility 
     * is not detected until run-time).
     * @return String interface version
     */

    public String getVersion() {
        return version;
    }

    /**
     * Display an info message to stdout
     * @param String message
     */

    public void infoMessage(String message) {
        stdlog_.debug("INFO: "+message);
    }

    /**
     * Display a warning message to stdout
     * @param String message
     */

    public void warningMessage(String message) {
        stdlog_.debug("WARNING: "+message);
    }

    /**
     * Display an error message to stdout
     * @param String message
     */

    public void errorMessage(String message) {
        stdlog_.debug("ERROR: "+message);
    }

    /**
     * Print in the report header.
     * This should be implemented by an extending class to print the
     * header of the report. It does not need to be called at the
     * the start of the report. It may be called at any time during 
     * the report and may be called multiple times.
     * @param message to be printed in the report header.
     */

    public void headerMessage(String message) {
        stdlog_.debug("REPORT HEADER: "+message);
    }

    /**
     * Print in the report footer.
     * This should be implemented by an extending class to print the
     * footer of the report. It does not need to be called at the
     * the end of the report. It may be called at any time during 
     * the report and may be called multiple times.
     * @param message to be printed in the report footer.
     */

    public void footerMessage(String message) {
        stdlog_.debug("REPORT FOOTER: "+message);
    }

    /**
     * Clear warning messages - does nothing in this class
     */

    public void clearWarnings() {};

    /**
     * Clear errors messages - does nothing in this class
     */

    public void clearErrors() {};

    /**
     * Get an instance of ScriptingUtilities class
     * @return ScriptingUtilities set by setUtils() call. If null, an
     * instance to a ScriptingUtilities class is returned.
     */

    public ScriptingUtilities getUtils() {
        if (scriptingUtils == null) {
            scriptingUtils = new ScriptingUtilities();
        }
        return scriptingUtils;
    }

    /**
     * Set the ScriptingUtilities object that is to be used by the 
     * scripting environment - used to over-ride the use of the
     * default ScriptingUtilities object.
     * @param ScriptingUtilities to be used in scripting environment
     */

    public void setUtils(ScriptingUtilities utils) {
        if (utils != null) {
            scriptingUtils = utils;
        }
    }

    /**
     * Get the number of Observation Blocks exposed to the scripting
     * environment. This method must be implemented.
     * @return array of ObservationBlockAPI objects
     */

    public abstract int getNumberOfObservationBlocks();

    /**
     * Get the ObservationBlock at the specified index (>=0)
     * This method must be implemented.
     * @param the index of the OB
     * @return the ObservationBlockAPI at that index, or null
     */

    public abstract ObservationBlockAPI getObservationBlockAt(int i);

}
